@@ -60,10 +60,10 @@ from .io import (  # noqa
     _jupyter_server_extension_paths, cache, ipywidget, serve, state,
 )
 from .layout import (  # noqa
-    Accordion, Card, Column, FlexBox, GridBox, GridSpec, Row, Spacer, Tabs,
-    WidgetBox,
+    Accordion, Card, Column, FlexBox, FloatPanel, GridBox, GridSpec, GridStack,
+    HSpacer, Row, Spacer, Swipe, Tabs, VSpacer, WidgetBox,
 )
-from .pane import Pane, panel  # noqa
+from .pane import panel  # noqa
 from .param import Param  # noqa
 from .template import Template  # noqa
 from .widgets import indicators, widget  # noqa
@@ -74,14 +74,17 @@ __all__ = (
     "Card",
     "Column",
     "FlexBox",
+    "FloatPanel",
     "GridBox",
     "GridSpec",
-    "Pane", # deprecated
+    "GridStack",
+    "HSpacer",
     "Param",
     "Row",
     "Spacer",
     "Tabs",
     "Template",
+    "VSpacer",
     "WidgetBox",
     "bind",
     "cache",
