@@ -12,14 +12,15 @@ from typing import (
 
 import numpy as np
 
+from bokeh.core.serialization import Serializer
 from bokeh.document import Document
 from bokeh.document.events import (
     ColumnDataChangedEvent, DocumentPatchedEvent, ModelChangedEvent,
 )
+from bokeh.document.json import PatchJson
 from bokeh.model import DataModel
-from bokeh.models import Box, ColumnDataSource, Model
-from bokeh.protocol import Protocol
-from typing_extensions import Literal
+from bokeh.models import ColumnDataSource, FlexBox, Model
+from bokeh.protocol.messages.patch_doc import patch_doc
 
 from .state import state
 
@@ -79,10 +80,13 @@ def diff(
     patch_events = [event for event in events if isinstance(event, DocumentPatchedEvent)]
     if not patch_events:
         return
-    monkeypatch_events(events)
-    msg_type: Literal["PATCH-DOC"] = "PATCH-DOC"
-    msg = Protocol().create(msg_type, patch_events, use_buffers=binary)
-    doc.callbacks._held_events = [e for e in doc.callbacks._held_events if e not in events]
+    monkeypatch_events(patch_events)
+    serializer = Serializer(references=doc.models.synced_references, deferred=binary)
+    patch_json = PatchJson(events=serializer.encode(patch_events))
+    header = patch_doc.create_header()
+    msg = patch_doc(header, {'use_buffers': binary}, patch_json)
+    doc.callbacks._held_events = [e for e in doc.callbacks._held_events if e not in patch_events]
+    doc.models.flush_synced(lambda model: not serializer.has_ref(model))
     return msg
 
 def remove_root(obj: 'Model', replace: Optional['Document'] = None) -> None:
@@ -169,7 +173,7 @@ def bokeh_repr(obj: 'Model', depth: int = 0, ignored: Optional[Iterable[str]] =
             v = v[:30] + '...'
         props.append('%s=%s' % (k, v))
     props_repr = ', '.join(props)
-    if isinstance(obj, Box):
+    if isinstance(obj, FlexBox):
         r += '{cls}(children=[\n'.format(cls=cls)
         for obj in obj.children: # type: ignore
             r += textwrap.indent(bokeh_repr(obj, depth=depth+1) + ',\n', '  ')
