@@ -1,18 +1,18 @@
 import json
 import os
-import pkg_resources
 import tempfile
 import traceback
 
+from runpy import run_path
 from unittest.mock import MagicMock
 from urllib.parse import parse_qs
 
 import param
 
-from runpy import run_path
 from tornado import web
 from tornado.wsgi import WSGIContainer
 
+from ..entry_points import entry_points_for
 from .state import state
 
 
@@ -76,14 +76,14 @@ class ParamHandler(BaseHandler):
             return
         args = parse_qs(self.request.query)
         params = self.deserialize(parameterized[0], args)
-        parameterized[0].param.set_param(**params)
+        parameterized[0].param.update(**params)
         self.set_header('Content-Type', 'application/json')
         self.write(self.serialize(parameterized[0], parameters))
 
 
 def build_tranquilize_application(files):
-    from tranquilizer.handler import ScriptHandler, NotebookHandler
-    from tranquilizer.main import make_app, UnsupportedFileType
+    from tranquilizer.handler import NotebookHandler, ScriptHandler
+    from tranquilizer.main import UnsupportedFileType, make_app
 
     functions = []
     for filename in files:
@@ -92,7 +92,7 @@ def build_tranquilize_application(files):
             source = ScriptHandler(filename)
         elif extension == 'ipynb':
             try:
-                import nbconvert # noqa
+                import nbconvert  # noqa
             except ImportError as e: # pragma no cover
                 raise ImportError("Please install nbconvert to serve Jupyter Notebooks.") from e
 
@@ -147,10 +147,10 @@ def param_rest_provider(files, endpoint):
             try:
                 run_path(filename)
             except Exception:
-                param.main.warning("Could not run app script on REST server startup.")
+                param.main.param.warning("Could not run app script on REST server startup.")
         elif extension == 'ipynb':
             try:
-                import nbconvert # noqa
+                import nbconvert  # noqa
             except ImportError:
                 raise ImportError("Please install nbconvert to serve Jupyter Notebooks.")
             from nbconvert import ScriptExporter
@@ -163,7 +163,7 @@ def param_rest_provider(files, endpoint):
                 try:
                     run_path(tmp.name, init_globals={'get_ipython': MagicMock()})
                 except Exception:
-                    param.main.warning("Could not run app notebook on REST server startup.")
+                    param.main.param.warning("Could not run app notebook on REST server startup.")
         else:
             raise ValueError('{} is not a script (.py) or notebook (.ipynb)'.format(filename))
 
@@ -178,5 +178,5 @@ REST_PROVIDERS = {
 }
 
 # Populate REST Providers from external extensions
-for entry_point in pkg_resources.iter_entry_points('panel.io.rest'):
-    REST_PROVIDERS[entry_point.name] = entry_point.resolve()
+for entry_point in entry_points_for('panel.io.rest'):
+    REST_PROVIDERS[entry_point.name] = entry_point.load()
