@@ -1,17 +1,22 @@
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, ClassVar, Mapping
+from typing import (
+    TYPE_CHECKING, Callable, ClassVar, List, Mapping,
+)
 
 import param
 
 from bokeh.models import Column as BkColumn, CustomJS
 
+from ..reactive import Reactive
 from .base import NamedListPanel
 from .card import Card
 
 if TYPE_CHECKING:
     from bokeh.model import Model
 
+    from ..viewable import Viewable
+
 
 class Accordion(NamedListPanel):
     """
@@ -49,9 +54,13 @@ class Accordion(NamedListPanel):
 
     _bokeh_model = BkColumn
 
-    _rename: ClassVar[Mapping[str, str | None]] = {'active': None, 'active_header_background': None,
-               'header_background': None, 'objects': 'children',
-               'dynamic': None, 'toggle': None, 'header_color': None}
+    _direction: ClassVar[str | None] = 'vertical'
+
+    _rename: ClassVar[Mapping[str, str | None]] = {
+        'active': None, 'active_header_background': None,
+        'header_background': None, 'objects': 'children',
+        'dynamic': None, 'toggle': None, 'header_color': None
+    }
 
     _toggle = """
     for (var child of accordion.children) {
@@ -63,7 +72,8 @@ class Accordion(NamedListPanel):
 
     _synced_properties = [
         'active_header_background', 'header_background', 'width',
-        'sizing_mode', 'width_policy', 'height_policy', 'header_color'
+        'sizing_mode', 'width_policy', 'height_policy', 'header_color',
+        'min_width', 'max_width'
     ]
 
     def __init__(self, *objects, **params):
@@ -72,6 +82,10 @@ class Accordion(NamedListPanel):
         self.param.watch(self._update_active, ['active'])
         self.param.watch(self._update_cards, self._synced_properties)
 
+    def _process_property_change(self, props):
+        props.pop('children', None)
+        return super()._process_property_change(props)
+
     def _get_objects(self, model, old_objects, doc, root, comm=None):
         """
         Returns new child models for the layout while reusing unchanged
@@ -80,10 +94,11 @@ class Accordion(NamedListPanel):
         from panel.pane.base import RerenderError, panel
         new_models = []
         if len(self._names) != len(self):
-            raise ValueError('Accordion names do not match objects, ensure '
-                             'that the Tabs.objects are not modified '
-                             'directly. Found %d names, expected %d.' %
-                             (len(self._names), len(self)))
+            raise ValueError(
+                'Accordion names do not match objects, ensure that the '
+                'Accordion.objects are not modified directly. Found '
+                f'{len(self._names)} names, expected {len(self)}.'
+            )
         for i, (name, pane) in enumerate(zip(self._names, self)):
             pane = panel(pane, name=name)
             self.objects[i] = pane
@@ -92,8 +107,10 @@ class Accordion(NamedListPanel):
             if obj not in self.objects:
                 self._panels[id(obj)]._cleanup(root)
 
-        params = {k: v for k, v in self.param.values().items()
-                  if k in self._synced_properties}
+        params = {
+            k: v for k, v in self.param.values().items()
+            if k in self._synced_properties
+        }
 
         ref = root.ref['id']
         current_objects = list(self)
@@ -102,15 +119,15 @@ class Accordion(NamedListPanel):
             params.update(self._apply_style(i))
             if id(pane) in self._panels:
                 card = self._panels[id(pane)]
+                card.param.update(**params)
             else:
                 card = Card(
                     pane, title=name, css_classes=['accordion'],
                     header_css_classes=['accordion-header'],
-                    margin=self.margin
+                    **params
                 )
                 card.param.watch(self._set_active, ['collapsed'])
                 self._panels[id(pane)] = card
-            card.param.update(**params)
             if ref in card._models:
                 panel = card._models[ref][0]
             else:
@@ -120,7 +137,9 @@ class Accordion(NamedListPanel):
                         cb = CustomJS(args={'accordion': model}, code=self._toggle)
                         panel.js_on_change('collapsed', cb)
                 except RerenderError:
-                    return self._get_objects(model, current_objects[:i], doc, root, comm)
+                    return self._get_objects(
+                        model, current_objects[:i], doc, root, comm
+                    )
             new_models.append(panel)
 
         self._updating_active = False
@@ -129,6 +148,10 @@ class Accordion(NamedListPanel):
         self._update_active()
         return new_models
 
+    def _compute_sizing_mode(self, children, props):
+        children = [subchild for child in children for subchild in child.children[1:]]
+        return super()._compute_sizing_mode(children, props)
+
     def _cleanup(self, root: Model | None = None) -> None:
         for panel in self._panels.values():
             panel._cleanup(root)
@@ -176,7 +199,21 @@ class Accordion(NamedListPanel):
             self._updating_active = False
 
     def _update_cards(self, *events):
-        params = {k: v for k, v in self.param.values().items()
-                  if k in self._synced_properties}
+        params = {
+            k: v for k, v in self.param.values().items()
+            if k in self._synced_properties
+        }
         for panel in self._panels.values():
             panel.param.update(**params)
+
+    # Public API
+
+    def select(
+        self, selector: type | Callable[[Viewable], bool] | None = None
+    ) -> List[Viewable]:
+        selected = Reactive.select(self, selector)
+        if self._panels:
+            for card in self._panels.values():
+                selected += card.select(selector)
+            return selected
+        return selected
