@@ -6,8 +6,8 @@ from __future__ import annotations
 
 from collections import defaultdict, namedtuple
 from typing import (
-    TYPE_CHECKING, Any, Dict, Iterable, Iterator, List, Mapping,
-    Optional, Tuple, Type
+    TYPE_CHECKING, Any, ClassVar, Dict, Iterable, Iterator, List, Mapping,
+    Optional, Tuple, Type,
 )
 
 import param
@@ -15,6 +15,7 @@ import param
 from bokeh.models import Column as BkColumn, Row as BkRow
 
 from ..io.model import hold
+from ..io.resources import CDN_DIST
 from ..io.state import state
 from ..reactive import Reactive
 from ..util import param_name, param_reprs
@@ -36,19 +37,19 @@ class Panel(Reactive):
     """
 
     # Used internally to optimize updates
-    _batch_update: bool = False
+    _batch_update: ClassVar[bool] = False
 
     # Bokeh model used to render this Panel
-    _bokeh_model: Type['Model'] | None = None
+    _bokeh_model: ClassVar[Type[Model]]
 
-    # Properties that should sync JS -> Python
-    _linked_props: List[str] = []
+    # Direction the layout flows in
+    _direction: ClassVar[str | None] = None
 
     # Parameters which require the preprocessors to be re-run
-    _preprocess_params: List[str] = []
+    _preprocess_params: ClassVar[List[str]] = []
 
     # Parameter -> Bokeh property renaming
-    _rename: Dict[str, str | None] = {'objects': 'children'}
+    _rename: ClassVar[Mapping[str, str | None]] = {'objects': 'children'}
 
     __abstract = True
 
@@ -77,14 +78,30 @@ class Panel(Reactive):
 
     def _update_model(
         self, events: Dict[str, param.parameterized.Event], msg: Dict[str, Any],
-        root: 'Model', model: 'Model', doc: 'Document', comm: Optional['Comm']
+        root: Model, model: Model, doc: Document, comm: Optional[Comm]
     ) -> None:
         msg = dict(msg)
-        inverse = {v: k for k, v in self._rename.items() if v is not None}
+        inverse = {v: k for k, v in self._property_mapping.items() if v is not None}
         preprocess = any(inverse.get(k, k) in self._preprocess_params for k in msg)
-        if self._rename['objects'] in msg:
+
+        # ALERT: Find a better way to handle this
+        if 'styles' in msg and root is model and 'overflow-x' in msg['styles']:
+            del msg['styles']['overflow-x']
+
+        obj_key = self._property_mapping['objects']
+        if obj_key in msg:
             old = events['objects'].old
-            msg[self._rename['objects']] = self._get_objects(model, old, doc, root, comm)
+            msg[obj_key] = children = self._get_objects(model, old, doc, root, comm)
+            msg.update(self._compute_sizing_mode(
+                children,
+                dict(
+                    sizing_mode=msg.get('sizing_mode', model.sizing_mode),
+                    styles=msg.get('styles', model.styles),
+                    width=msg.get('width', model.width),
+                    min_width=msg.get('min_width', model.min_width),
+                    margin=msg.get('margin', model.margin)
+                )
+            ))
 
         with hold(doc):
             update = Panel._batch_update
@@ -105,14 +122,14 @@ class Panel(Reactive):
     #----------------------------------------------------------------
 
     def _get_objects(
-        self, model: 'Model', old_objects: List['Viewable'], doc: 'Document',
-        root: 'Model', comm: Optional['Comm'] = None
+        self, model: Model, old_objects: List[Viewable], doc: Document,
+        root: Model, comm: Optional[Comm] = None
     ):
         """
         Returns new child models for the layout while reusing unchanged
         models and cleaning up any dropped objects.
         """
-        from ..pane.base import panel, RerenderError
+        from ..pane.base import RerenderError, panel
         new_models = []
         for i, pane in enumerate(self.objects):
             pane = panel(pane)
@@ -123,8 +140,9 @@ class Panel(Reactive):
                 obj._cleanup(root)
 
         current_objects = list(self.objects)
+        ref = root.ref['id']
         for i, pane in enumerate(self.objects):
-            if pane in old_objects:
+            if pane in old_objects and ref in pane._models:
                 child, _ = pane._models[root.ref['id']]
             else:
                 try:
@@ -135,25 +153,135 @@ class Panel(Reactive):
         return new_models
 
     def _get_model(
-        self, doc: Document, root: Optional['Model'] = None,
-        parent: Optional['Model'] = None, comm: Optional[Comm] = None
-    ) -> 'Model':
+        self, doc: Document, root: Optional[Model] = None,
+        parent: Optional[Model] = None, comm: Optional[Comm] = None
+    ) -> Model:
         if self._bokeh_model is None:
             raise ValueError(f'{type(self).__name__} did not define a _bokeh_model.')
         model = self._bokeh_model()
-        if root is None:
-            root = model
-        objects = self._get_objects(model, [], doc, root, comm)
-        props = dict(self._init_params(), objects=objects)
-        model.update(**self._process_param_change(props))
+        root = root or model
         self._models[root.ref['id']] = (model, parent)
-        self._link_props(model, self._linked_props, doc, root, comm)
+        objects = self._get_objects(model, [], doc, root, comm)
+        props = self._get_properties(doc)
+        props[self._property_mapping['objects']] = objects
+        props.update(self._compute_sizing_mode(objects, props))
+        model.update(**props)
+        self._link_props(model, self._linked_properties, doc, root, comm)
         return model
 
+    def _compute_sizing_mode(self, children, props):
+        """
+        Handles inference of correct layout sizing mode by inspecting
+        the children and adapting to their layout properties. This
+        aims to provide a layer of backward compatibility for the
+        layout behavior before v1.0 and provide general usability
+        improvements.
+
+        The code iterates over the children and extracts their sizing_mode,
+        width and height. Based on these values we infer a few overrides
+        for the container sizing_mode, width and height:
+
+        - If a child is responsive in width then the container should
+          also be responsive in width (unless it has a fixed size).
+        - If a container is vertical (e.g. a Column) and a child is
+          responsive in height then the container should also be
+          responsive.
+        - If a container is horizontal (e.g. a Row) and all children
+          are responsive in height then the container should also be
+          responsive. This behavior is asymmetrical with height
+          because there isn't always vertical space to expand into
+          and it is better for the component to match the height of
+          the other children.
+        - Always compute the fixed sizes of the children (if available)
+          and provide this as min_width and min_height settings to
+          ensure sufficient space is available.
+        """
+        margin = props.get('margin', self.margin)
+        sizing_mode = props.get('sizing_mode', self.sizing_mode)
+        if sizing_mode == 'fixed':
+            return {}
+
+        # Iterate over children and determine responsiveness along
+        # each axis, scaling and the widths of each component.
+        heights, widths = [], []
+        all_expand_width, all_expand_height, expand_width, expand_height, scale = True, True, False, False, False
+        for child in children:
+            if child.sizing_mode and 'scale' in child.sizing_mode:
+                scale = True
+            if child.sizing_mode in ('stretch_width', 'stretch_both', 'scale_width', 'scale_both'):
+                expand_width = True
+            else:
+                width = child.width or child.min_width
+                if width:
+                    if isinstance(margin, tuple):
+                        if len(margin) == 2:
+                            width += margin[1]*2
+                        else:
+                            width += margin[1] + margin[3]
+                    else:
+                        width += margin*2
+                    widths.append(width)
+                all_expand_width = False
+            if child.sizing_mode in ('stretch_height', 'stretch_both', 'scale_height', 'scale_both'):
+                expand_height = True
+            else:
+                height = child.height or child.min_height
+                if height:
+                    if isinstance(margin, tuple):
+                        if len(margin) == 2:
+                            height += margin[0]*2
+                        else:
+                            height += margin[0] + margin[2]
+                    else:
+                        height += margin*2
+                    heights.append(height)
+                all_expand_height = False
+
+        # Infer new sizing mode based on children
+        mode = 'scale' if scale else 'stretch'
+        if self._direction == 'horizontal':
+            allow_height_scale = all_expand_height
+        else:
+            allow_height_scale = True
+        if expand_width and expand_height and not self.width and not self.height:
+            if allow_height_scale or 'both' in (sizing_mode or ''):
+                sizing_mode = f'{mode}_both'
+            else:
+                sizing_mode = f'{mode}_width'
+        elif expand_width and not self.width:
+            sizing_mode = f'{mode}_width'
+        elif expand_height and not self.height and allow_height_scale:
+            sizing_mode = f'{mode}_height'
+        if sizing_mode is None:
+            return {'sizing_mode': props.get('sizing_mode')}
+
+        properties = {'sizing_mode': sizing_mode}
+        if ((sizing_mode.endswith('_width') or sizing_mode.endswith('_both')) and
+            not all_expand_width and widths and 'min_width' not in properties):
+            width_op = max if self._direction == 'vertical' else sum
+            min_width = width_op(widths)
+            properties['min_width'] = min(min_width, properties.get('max_width') or 0)
+        if ((sizing_mode.endswith('_height') or sizing_mode.endswith('_both')) and
+            not all_expand_height and heights and 'min_height' not in properties):
+            height_op = max if self._direction == 'horizontal' else sum
+            min_height = height_op(heights)
+            properties['min_height'] = min(min_height, properties.get('max_height') or 0)
+        return properties
+
     #----------------------------------------------------------------
     # Public API
     #----------------------------------------------------------------
 
+    def get_root(
+        self, doc: Optional[Document] = None, comm: Optional[Comm] = None,
+        preprocess: bool = True
+    ) -> Model:
+        root = super().get_root(doc, comm, preprocess)
+        # ALERT: Find a better way to handle this
+        if hasattr(root, 'styles') and 'overflow-x' in root.styles:
+            del root.styles['overflow-x']
+        return root
+
     def select(self, selector=None):
         """
         Iterates over the Viewable and any potential children in the
@@ -180,15 +308,15 @@ class ListLike(param.Parameterized):
     objects = param.List(default=[], doc="""
         The list of child objects that make up the layout.""")
 
-    _preprocess_params: List[str] = ['objects']
+    _preprocess_params: ClassVar[List[str]] = ['objects']
 
-    def __getitem__(self, index: int | slice) -> 'Viewable' | List['Viewable']:
+    def __getitem__(self, index: int | slice) -> Viewable | List[Viewable]:
         return self.objects[index]
 
     def __len__(self) -> int:
         return len(self.objects)
 
-    def __iter__(self) -> Iterator['Viewable']:
+    def __iter__(self) -> Iterator[Viewable]:
         for obj in self.objects:
             yield obj
 
@@ -210,7 +338,7 @@ class ListLike(param.Parameterized):
             other = list(other)
         return self.clone(*(other+self.objects))
 
-    def __contains__(self, obj: 'Viewable') -> bool:
+    def __contains__(self, obj: Viewable) -> bool:
         return obj in self.objects
 
     def __setitem__(self, index: int | slice, panes: Iterable[Any]) -> None:
@@ -322,7 +450,7 @@ class ListLike(param.Parameterized):
         new_objects.insert(index, panel(obj))
         self.objects = new_objects
 
-    def pop(self, index: int) -> 'Viewable':
+    def pop(self, index: int) -> Viewable:
         """
         Pops an item from the layout by index.
 
@@ -335,7 +463,7 @@ class ListLike(param.Parameterized):
         self.objects = new_objects
         return obj
 
-    def remove(self, obj: 'Viewable') -> None:
+    def remove(self, obj: Viewable) -> None:
         """
         Removes an object from the layout.
 
@@ -361,9 +489,9 @@ class NamedListLike(param.Parameterized):
     objects = param.List(default=[], doc="""
         The list of child objects that make up the layout.""")
 
-    _preprocess_params = ['objects']
+    _preprocess_params: ClassVar[List[str]] = ['objects']
 
-    def __init__(self, *items: List[Any, Tuple[str, Any]], **params: Any):
+    def __init__(self, *items: List[Any | Tuple[str, Any]], **params: Any):
         if 'objects' in params:
             if items:
                 raise ValueError('%s objects should be supplied either '
@@ -416,13 +544,13 @@ class NamedListLike(param.Parameterized):
     # Public API
     #----------------------------------------------------------------
 
-    def __getitem__(self, index) -> 'Viewable' | List['Viewable']:
+    def __getitem__(self, index) -> Viewable | List[Viewable]:
         return self.objects[index]
 
     def __len__(self) -> int:
         return len(self.objects)
 
-    def __iter__(self) -> Iterator['Viewable']:
+    def __iter__(self) -> Iterator[Viewable]:
         for obj in self.objects:
             yield obj
 
@@ -563,7 +691,7 @@ class NamedListLike(param.Parameterized):
         self._names.insert(index, new_name)
         self.objects = new_objects
 
-    def pop(self, index: int) -> 'Viewable':
+    def pop(self, index: int) -> Viewable:
         """
         Pops an item from the tabs by index.
 
@@ -577,7 +705,7 @@ class NamedListLike(param.Parameterized):
         self.objects = new_objects
         return obj
 
-    def remove(self, pane: 'Viewable') -> None:
+    def remove(self, pane: Viewable) -> None:
         """
         Removes an object from the tabs.
 
@@ -607,16 +735,13 @@ class ListPanel(ListLike, Panel):
     An abstract baseclass for Panel objects with list-like children.
     """
 
-    margin = param.Parameter(default=0, doc="""
-        Allows to create additional space around the component. May
-        be specified as a two-tuple of the form (vertical, horizontal)
-        or a four-tuple (top, right, bottom, left).""")
-
     scroll = param.Boolean(default=False, doc="""
         Whether to add scrollbars if the content overflows the size
         of the container.""")
 
-    _source_transforms: Mapping[str, str | None] = {'scroll': None}
+    _rename: ClassVar[Mapping[str, str | None]] = {'scroll': None}
+
+    _source_transforms: ClassVar[Mapping[str, str | None]] = {'scroll': None}
 
     __abstract = True
 
@@ -632,6 +757,13 @@ class ListPanel(ListLike, Panel):
             params['objects'] = [panel(pane) for pane in params['objects']]
         super(Panel, self).__init__(**params)
 
+    @property
+    def _linked_properties(self):
+        return tuple(
+            self._property_mapping.get(p, p) for p in self.param
+            if p not in ListPanel.param and self._property_mapping.get(p, p) is not None
+        )
+
     def _process_param_change(self, params: Dict[str, Any]) -> Dict[str, Any]:
         scroll = params.pop('scroll', None)
         css_classes = self.css_classes or []
@@ -641,7 +773,7 @@ class ListPanel(ListLike, Panel):
             params['css_classes'] = css_classes
         return super()._process_param_change(params)
 
-    def _cleanup(self, root: 'Model' | None):
+    def _cleanup(self, root: Model | None = None) -> None:
         if root is not None and root.ref['id'] in state._fake_roots:
             state._fake_roots.remove(root.ref['id'])
         super()._cleanup(root)
@@ -654,29 +786,26 @@ class NamedListPanel(NamedListLike, Panel):
     active = param.Integer(default=0, bounds=(0, None), doc="""
         Index of the currently displayed objects.""")
 
-    margin = param.Parameter(default=0, doc="""
-        Allows to create additional space around the component. May
-        be specified as a two-tuple of the form (vertical, horizontal)
-        or a four-tuple (top, right, bottom, left).""")
-
     scroll = param.Boolean(default=False, doc="""
         Whether to add scrollbars if the content overflows the size
         of the container.""")
 
-    _source_transforms: Dict[str, str | None] = {'scroll': None}
+    _rename: ClassVar[Mapping[str, str | None]] = {'scroll': None}
+
+    _source_transforms: ClassVar[Mapping[str, str | None]] = {'scroll': None}
 
     __abstract = True
 
     def _process_param_change(self, params: Dict[str, Any]) -> Dict[str, Any]:
-        scroll = params.pop('scroll', None)
-        css_classes = self.css_classes or []
-        if scroll:
-            params['css_classes'] = css_classes + ['scrollable']
-        elif scroll == False:
+        if 'scroll' in params:
+            scroll = params.pop('scroll')
+            css_classes = list(self.css_classes or [])
+            if scroll:
+                css_classes += ['scrollable']
             params['css_classes'] = css_classes
         return super()._process_param_change(params)
 
-    def _cleanup(self, root: 'Model' | None) -> None:
+    def _cleanup(self, root: Model | None = None) -> None:
         if root is not None and root.ref['id'] in state._fake_roots:
             state._fake_roots.remove(root.ref['id'])
         super()._cleanup(root)
@@ -700,11 +829,11 @@ class Row(ListPanel):
     >>> pn.Row(some_widget, some_pane, some_python_object)
     """
 
-    col_sizing = param.Parameter()
+    _bokeh_model: ClassVar[Type[Model]] = BkRow
 
-    _bokeh_model: Type['Model'] = BkRow
+    _direction = 'horizontal'
 
-    _rename = dict(ListPanel._rename, col_sizing='cols')
+    _stylesheets: ClassVar[list[str]] = [f'{CDN_DIST}css/listpanel.css']
 
 
 class Column(ListPanel):
@@ -723,11 +852,11 @@ class Column(ListPanel):
     >>> pn.Column(some_widget, some_pane, some_python_object)
     """
 
-    row_sizing = param.Parameter()
+    _bokeh_model: ClassVar[Type[Model]] = BkColumn
 
-    _bokeh_model: Type['Model'] = BkColumn
+    _direction = 'vertical'
 
-    _rename = dict(ListPanel._rename, row_sizing='rows')
+    _stylesheets: ClassVar[list[str]] = [f'{CDN_DIST}css/listpanel.css']
 
 
 class WidgetBox(ListPanel):
@@ -759,24 +888,31 @@ class WidgetBox(ListPanel):
         Whether to lay out the widgets in a Row layout as opposed
         to a Column layout.""")
 
-    margin = param.Parameter(default=5, doc="""
-        Allows to create additional space around the component. May
-        be specified as a two-tuple of the form (vertical, horizontal)
-        or a four-tuple (top, right, bottom, left).""")
+    _source_transforms: ClassVar[Mapping[str, str | None]] = {
+        'disabled': None, 'horizontal': None
+    }
 
-    _source_transforms = {'disabled': None, 'horizontal': None}
+    _rename: ClassVar[Mapping[str, str | None]] = {
+        'disabled': None, 'objects': 'children', 'horizontal': None
+    }
 
-    _rename = {'objects': 'children', 'horizontal': None}
+    _stylesheets: ClassVar[list[str]] = [
+        f'{CDN_DIST}css/widgetbox.css', f'{CDN_DIST}css/listpanel.css'
+    ]
 
     @property
-    def _bokeh_model(self) -> Type['Model']: # type: ignore
+    def _bokeh_model(self) -> Type[Model]: # type: ignore
         return BkRow if self.horizontal else BkColumn
 
+    @property
+    def _direction(self):
+        return 'vertical' if self.horizontal else 'vertical'
+
     @param.depends('disabled', 'objects', watch=True)
     def _disable_widgets(self) -> None:
-        for obj in self:
-            if hasattr(obj, 'disabled'):
-                obj.disabled = self.disabled
+        from ..widgets import Widget
+        for obj in self.select(Widget):
+            obj.disabled = self.disabled
 
     def __init__(self, *objects: Any, **params: Any):
         super().__init__(*objects, **params)
