@@ -107,7 +107,7 @@ class Callback(param.Parameterized):
         named parameters to the callback.""")
 
     code = param.Dict(default=None, doc="""
-        A dictionary mapping from a source specication to a JS code
+        A dictionary mapping from a source specification to a JS code
         snippet to be executed if the source property changes.""")
 
     # Mapping from a source id to a Link instance
@@ -447,7 +447,7 @@ class CallbackGenerator(object):
                     if isinstance(v, BkModel) and k not in references:
                         references[k] = v
 
-            if isinstance(target, HoloViews):
+            if isinstance(target, HoloViews) and ref in target._plots:
                 tgt = target._plots[ref][0]
             else:
                 tgt = target
