@@ -6,11 +6,12 @@ export function transform_cds_to_records(cds: ColumnDataSource, addId: boolean =
   const cdsLength = cds.get_length()
   if (columns.length === 0||cdsLength === null)
     return []
+
   for (let i = start; i < cdsLength; i++) {
     const item: any = {}
     for (const column of columns) {
       let array: any = cds.get_array(column);
-      const shape = array[0].shape == null ? null : array[0].shape;
+      const shape = (array[0] == null || array[0].shape == null) ? null : array[0].shape;
       if ((shape != null) && (shape.length > 1) && (typeof shape[0] == "number"))
         item[column] = array.slice(i*shape[1], i*shape[1]+shape[1])
       else
