@@ -1,5 +1,6 @@
 export {AcePlot} from "./ace"
 export {Audio} from "./audio"
+export {BrowserInfo} from "./browser"
 export {Card} from "./card"
 export {CommManager} from "./comm_manager"
 export {CustomSelect} from "./customselect"
@@ -8,7 +9,6 @@ export {DatetimePicker} from "./datetime_picker"
 export {DeckGLPlot} from "./deckgl"
 export {ECharts} from "./echarts"
 export {HTML} from "./html"
-export {IDOM} from "./idom"
 export {IPyWidget} from "./ipywidget"
 export {JSON} from "./json"
 export {JSONEditor} from "./jsoneditor"
@@ -16,6 +16,7 @@ export {FileDownload} from "./file_download"
 export {KaTeX} from "./katex"
 export {Location} from "./location"
 export {MathJax} from "./mathjax"
+export {PDF} from "./pdf"
 export {Perspective} from "./perspective"
 export {Player} from "./player"
 export {PlotlyPlot} from "./plotly"
@@ -32,4 +33,5 @@ export {TrendIndicator} from "./trend"
 export {VegaPlot} from "./vega"
 export {Video} from "./video"
 export {VideoStream} from "./videostream"
+export {VizzuChart} from "./vizzu"
 export * from "./vtk"
