@@ -55,6 +55,12 @@ export class VTKVolumePlotView extends AbstractVTKView {
       this.volume.getProperty().setDiffuse(this.model.diffuse)
       this._vtk_renwin.getRenderWindow().render()
     })
+    this.connect(this.model.properties.camera.change, () => {
+      if (!this._setting_camera) {
+	this._set_camera_state()
+	this._vtk_renwin.getRenderWindow().render()
+      }
+    })
     this.connect(this.model.properties.specular.change, () => {
       this.volume.getProperty().setSpecular(this.model.specular)
       this._vtk_renwin.getRenderWindow().render()
@@ -103,6 +109,12 @@ export class VTKVolumePlotView extends AbstractVTKView {
       if (this._controllerWidget != null)
         this._controllerWidget.setExpanded(this.model.controller_expanded)
     })
+    this.connect(this.model.properties.nan_opacity.change, () => {
+      const scalar_opacity = this.image_actor_i.getProperty().getScalarOpacity()
+      scalar_opacity.get(["nodes"]).nodes[0].y = this.model.nan_opacity
+      scalar_opacity.modified()
+      this._vtk_renwin.getRenderWindow().render()
+    })
   }
 
   render(): void {
@@ -112,9 +124,8 @@ export class VTKVolumePlotView extends AbstractVTKView {
     super.render()
     this._create_orientation_widget()
     this._set_axes()
-    if (!this.model.camera)
-      this._vtk_renwin.getRenderer().resetCamera()
-    else
+    this._vtk_renwin.getRenderer().resetCamera()
+    if (Object.keys(this.model.camera).length)
       this._set_camera_state()
     this._get_camera_state()
   }
@@ -126,7 +137,7 @@ export class VTKVolumePlotView extends AbstractVTKView {
 
   init_vtk_renwin(): void {
     this._vtk_renwin = vtkns.FullScreenRenderWindow.newInstance({
-      rootContainer: this.el,
+      rootContainer: this.shadow_el,
       container: this._vtk_container,
     })
   }
@@ -263,9 +274,13 @@ export class VTKVolumePlotView extends AbstractVTKView {
 
     // set_color and opacity
     const piecewiseFunction = vtkns.PiecewiseFunction.newInstance()
-    piecewiseFunction.removeAllPoints()
-    piecewiseFunction.addPoint(0, 1)
     const lookupTable = this.volume.getProperty().getRGBTransferFunction(0)
+    const range = this.volume.getMapper().getInputData().getPointData().getScalars().getRange()
+    piecewiseFunction.removeAllPoints()
+    piecewiseFunction.addPoint(range[0]-1, this.model.nan_opacity)
+    piecewiseFunction.addPoint(range[0], 1)
+    piecewiseFunction.addPoint(range[1], 1)
+
     const property = image_actor_i.getProperty()
     image_actor_j.setProperty(property)
     image_actor_k.setProperty(property)
@@ -293,6 +308,10 @@ export class VTKVolumePlotView extends AbstractVTKView {
     const dataRange = dataArray.getRange()
 
     const lookupTable = vtkns.ColorTransferFunction.newInstance()
+    if (this.model.colormap != null) {
+      const preset = vtkns.ColorTransferFunction.vtkColorMaps.getPresetByName(this.model.colormap);
+      lookupTable.applyColorMap(preset);
+    }
     lookupTable.onModified(
       () => (this.model.mapper = vtkLutToMapper(lookupTable))
     )
@@ -360,10 +379,9 @@ export class VTKVolumePlotView extends AbstractVTKView {
   }
 
   _set_slices_visibility(visibility: boolean): void {
-    this._vtk_renwin
-      .getRenderer()
-      .getActors()
-      .map((actor: any) => actor.setVisibility(visibility))
+    this.image_actor_i.setVisibility(visibility)
+    this.image_actor_j.setVisibility(visibility)
+    this.image_actor_k.setVisibility(visibility)
   }
 
   _set_volume_visibility(visibility: boolean): void {
@@ -382,6 +400,7 @@ export namespace VTKVolumePlot {
     edge_gradient: p.Property<number>
     interpolation: p.Property<Interpolation>
     mapper: p.Property<ColorMapper>
+    nan_opacity: p.Property<number>
     render_background: p.Property<string>
     rescale: p.Property<boolean>
     sampling: p.Property<number>
@@ -404,7 +423,7 @@ export class VTKVolumePlot extends AbstractVTKPlot {
     super(attrs)
   }
 
-  static init_VTKVolumePlot(): void {
+  static {
     this.prototype.default_view = VTKVolumePlotView
 
     this.define<VTKVolumePlot.Props>(({Any, Array, Boolean, Int, Number, String, Struct}) => ({
@@ -416,7 +435,8 @@ export class VTKVolumePlot extends AbstractVTKPlot {
       display_volume:       [ Boolean,          true ],
       edge_gradient:        [ Number,            0.2 ],
       interpolation:        [ Interpolation, 'fast_linear'],
-      mapper:               [ Struct({palette: Array(String), low: Number, high: Number}) ],
+      mapper:               [ Struct({palette: Array(String), low: Number, high: Number}), {palette: [], low: 0, high: 0} ],
+      nan_opacity:          [ Number,              1 ],
       render_background:    [ String,      '#52576e' ],
       rescale:              [ Boolean,         false ],
       sampling:             [ Number,            0.4 ],
@@ -428,5 +448,10 @@ export class VTKVolumePlot extends AbstractVTKPlot {
       specular_power:       [ Number,            8.0 ],
       controller_expanded:  [ Boolean,          true ],
     }))
+
+    this.override<VTKVolumePlot.Props>({
+      height: 300,
+      width: 300,
+    })
   }
 }
