@@ -9,7 +9,7 @@ import sys
 from collections import OrderedDict, defaultdict
 from functools import partial
 from typing import (
-    TYPE_CHECKING, Any, ClassVar, Mapping, Optional, Type,
+    TYPE_CHECKING, Any, ClassVar, Mapping, Optional, Tuple, Type,
 )
 
 import param
@@ -20,11 +20,11 @@ from packaging.version import Version
 
 from ..io import state, unlocked
 from ..layout import (
-    Column, HSpacer, Row, VSpacer, WidgetBox,
+    Column, HSpacer, Row, WidgetBox,
 )
 from ..viewable import Layoutable, Viewable
 from ..widgets import Player
-from .base import Pane, PaneBase, RerenderError
+from .base import PaneBase, RerenderError, panel
 from .plot import Bokeh, Matplotlib
 from .plotly import Plotly
 
@@ -104,6 +104,11 @@ class HoloViews(PaneBase):
 
     _rerender_params = ['object', 'backend']
 
+    _skip_layoutable = (
+        'background', 'css_classes', 'margin', 'name', 'sizing_mode',
+        'width', 'height', 'max_width', 'max_height'
+    )
+
     def __init__(self, object=None, **params):
         self._initialized = False
         self._responsive_content = False
@@ -118,59 +123,56 @@ class HoloViews(PaneBase):
             if p in Layoutable.param and v != self.param[p].default
         ]
         watcher = self.param.watch(self._update_widgets, self._rerender_params)
-        self._callbacks.append(watcher)
+        self._internal_callbacks.append(watcher)
         self._initialized = True
         self._update_responsive()
         self._update_widgets()
 
     def _param_change(self, *events: param.parameterized.Event) -> None:
+        if self._object_changing:
+            return
         self._track_overrides(*(e for e in events if e.name in Layoutable.param))
-        super()._param_change(*(e for e in events if e.name in self._overrides))
+        super()._param_change(*(e for e in events if e.name in self._overrides+['css_classes']))
 
     @param.depends('center', 'widget_location', watch=True)
     def _update_layout(self):
         loc = self.widget_location
-        if not len(self.widget_box):
-            widgets = []
-        elif loc in ('left', 'right'):
-            widgets = Column(VSpacer(), self.widget_box, VSpacer())
-        elif loc in ('top', 'bottom'):
-            widgets = Row(HSpacer(), self.widget_box, HSpacer())
-        elif loc in ('top_left', 'bottom_left'):
-            widgets = Row(self.widget_box, HSpacer())
-        elif loc in ('top_right', 'bottom_right'):
-            widgets = Row(HSpacer(), self.widget_box)
-        elif loc in ('left_top', 'right_top'):
-            widgets = Column(self.widget_box, VSpacer())
-        elif loc in ('left_bottom', 'right_bottom'):
-            widgets = Column(VSpacer(), self.widget_box)
-
         center = self.center and not self._responsive_content
+        alignments = {
+            'left': (Row, ('start', 'center'), True),
+            'right': (Row, ('end', 'center'), False),
+            'top': (Column, ('center', 'start'), True),
+            'bottom': (Column, ('center', 'end'), False),
+            'top_left': (Column, 'start', True),
+            'top_right': (Column, ('end', 'start'), True),
+            'bottom_left': (Column, ('start', 'end'), False),
+            'bottom_right': (Column, 'end', False),
+            'left_top': (Row, 'start', True),
+            'left_bottom': (Row, ('start', 'end'), True),
+            'right_top': (Row, ('end', 'start'), False),
+            'right_bottom': (Row, 'end', False)
+        }
 
-        self._widget_container = widgets
-        if not widgets:
+        layout, align, widget_first = alignments[loc]
+        self.widget_box.align = align
+        self._widget_container = self.widget_box
+        if not len(self.widget_box):
             if center:
                 components = [HSpacer(), self, HSpacer()]
             else:
                 components = [self]
-        elif center:
-            if loc.startswith('left'):
-                components = [widgets, HSpacer(), self, HSpacer()]
-            elif loc.startswith('right'):
-                components = [HSpacer(), self, HSpacer(), widgets]
-            elif loc.startswith('top'):
-                components = [HSpacer(), Column(widgets, Row(HSpacer(), self, HSpacer())), HSpacer()]
-            elif loc.startswith('bottom'):
-                components = [HSpacer(), Column(Row(HSpacer(), self, HSpacer()), widgets), HSpacer()]
+            self.layout[:] = components
+            return
+
+        items = (self.widget_box, self) if widget_first else (self, self.widget_box)
+        if not center:
+            components = [layout(*items)]
+        elif layout is Column:
+            components = [HSpacer(), layout(*items), HSpacer()]
+        elif loc.startswith('left'):
+            components = [self.widget_box, HSpacer(), self, HSpacer()]
         else:
-            if loc.startswith('left'):
-                components = [widgets, self]
-            elif loc.startswith('right'):
-                components = [self, widgets]
-            elif loc.startswith('top'):
-                components = [Column(widgets, self)]
-            elif loc.startswith('bottom'):
-                components = [Column(self, widgets)]
+            components = [HSpacer(), self, HSpacer(), self.widget_box]
         self.layout[:] = components
 
     #----------------------------------------------------------------
@@ -239,15 +241,15 @@ class HoloViews(PaneBase):
         self._values = values
 
         # Clean up anything models listening to the previous widgets
-        for cb in list(self._callbacks):
+        for cb in list(self._internal_callbacks):
             if cb.inst in self.widget_box.objects:
                 cb.inst.param.unwatch(cb)
-                self._callbacks.remove(cb)
+                self._internal_callbacks.remove(cb)
 
         # Add new widget callbacks
         for widget in widgets:
             watcher = widget.param.watch(self._widget_callback, 'value')
-            self._callbacks.append(watcher)
+            self._internal_callbacks.append(watcher)
 
         self.widget_box[:] = widgets
         if ((widgets and self.widget_box not in self._widget_container) or
@@ -333,8 +335,12 @@ class HoloViews(PaneBase):
         self._syncing_props = True
         try:
             self.param.update({k: v for k, v in params.items() if k not in self._overrides})
+            if backend != 'bokeh':
+                return
+            plot_props = plot.state.properties()
             props = {
                 o: getattr(self, o) for o in self._overrides
+                if o in plot_props
             }
             if props:
                 plot.state.update(**props)
@@ -385,6 +391,10 @@ class HoloViews(PaneBase):
 
         kwargs = {p: v for p, v in self.param.values().items()
                   if p in Layoutable.param and p != 'name'}
+        if self.sizing_mode and (self.sizing_mode.endswith('width') or self.sizing_mode.endswith('both')):
+            del kwargs['width']
+        if self.sizing_mode and (self.sizing_mode.endswith('height') or self.sizing_mode.endswith('both')):
+            del kwargs['height']
         child_pane = self._get_pane(backend, state, **kwargs)
         self._update_plot(plot, child_pane)
         model = child_pane._get_model(doc, root, parent, comm)
@@ -397,7 +407,7 @@ class HoloViews(PaneBase):
         return model
 
     def _get_pane(self, backend, state, **kwargs):
-        pane_type = self._panes.get(backend, Pane)
+        pane_type = self._panes.get(backend, panel)
         if isinstance(pane_type, type):
             if issubclass(pane_type, Matplotlib):
                 kwargs['tight'] = True
@@ -427,6 +437,8 @@ class HoloViews(PaneBase):
                 params['theme'] = self.theme
             elif doc.theme and getattr(doc.theme, '_json') != {'attrs': {}}:
                 params['theme'] = doc.theme
+            elif self._design.theme.bokeh_theme:
+                params['theme'] = self._design.theme.bokeh_theme
             if mode != renderer.mode:
                 params['mode'] = mode
             if params:
@@ -464,7 +476,7 @@ class HoloViews(PaneBase):
             return False
         from holoviews.core.dimension import Dimensioned
         from holoviews.plotting.plot import Plot
-        return isinstance(obj, Dimensioned) or isinstance(obj, Plot)
+        return isinstance(obj, (Dimensioned, Plot))
 
     def jslink(self, target, code=None, args=None, bidirectional=False, **links):
         if links and code:
@@ -614,6 +626,8 @@ class Interactive(PaneBase):
 
     priority: ClassVar[float | bool | None] = None
 
+    _ignored_refs: ClassVar[Tuple[str]] = ['object']
+
     def __init__(self, object=None, **params):
         super().__init__(object, **params)
         self._update_layout()
