@@ -8,7 +8,7 @@ import sys
 from contextlib import contextmanager
 from io import BytesIO
 from typing import (
-    TYPE_CHECKING, Any, ClassVar, Mapping, Optional,
+    TYPE_CHECKING, Any, ClassVar, Dict, Mapping, Optional,
 )
 
 import param
@@ -23,7 +23,9 @@ from ..io.notebook import push
 from ..util import escape
 from ..viewable import Layoutable
 from .base import PaneBase
-from .image import PNG
+from .image import (
+    PDF, PNG, SVG, Image,
+)
 from .ipywidget import IPyWidget
 from .markup import HTML
 
@@ -40,7 +42,7 @@ def _wrap_callback(cb, wrapped, doc, comm, callbacks):
     Wraps a bokeh callback ensuring that any events triggered by it
     appropriately dispatch events in the notebook. Also temporarily
     replaces the wrapped callback with the real one while the callback
-    is exectuted to ensure the callback can be removed as usual.
+    is executed to ensure the callback can be removed as usual.
     """
     hold = doc.callbacks.hold_value
     doc.hold('combine')
@@ -76,7 +78,9 @@ class Bokeh(PaneBase):
 
     priority: ClassVar[float | bool | None] = 0.8
 
-    _rename: ClassVar[Mapping[str, str | None]] = {'autodispatch': None, 'theme': None}
+    _rename: ClassVar[Mapping[str, str | None]] = {
+        'autodispatch': None, 'theme': None
+    }
 
     def __init__(self, object=None, **params):
         super().__init__(object, **params)
@@ -88,7 +92,7 @@ class Bokeh(PaneBase):
 
     def _param_change(self, *events: param.parameterized.Event) -> None:
         self._track_overrides(*(e for e in events if e.name in Layoutable.param))
-        super()._param_change(*(e for e in events if e.name in self._overrides))
+        super()._param_change(*(e for e in events if e.name in self._overrides+['css_classes']))
 
     @classmethod
     def applies(cls, obj: Any) -> float | bool | None:
@@ -179,13 +183,13 @@ class Bokeh(PaneBase):
 
         self._models[ref] = (model, parent)
 
-        if self.theme:
-            doc.theme = self.theme
+        # Apply theme
+        self._design.apply_bokeh_theme_to_model(model, self.theme)
 
         return model
 
 
-class Matplotlib(PNG, IPyWidget):
+class Matplotlib(Image, IPyWidget):
     """
     The `Matplotlib` pane allows displaying any displayable Matplotlib figure
     inside a Panel app.
@@ -206,6 +210,12 @@ class Matplotlib(PNG, IPyWidget):
     dpi = param.Integer(default=144, bounds=(1, None), doc="""
         Scales the dpi of the matplotlib figure.""")
 
+    encode = param.Boolean(default=True, doc="""
+        Whether to encode SVG out as base64.""")
+
+    format = param.Selector(default='png', objects=['png', 'svg'], doc="""
+        The format to render the plot as if the plot is not interactive.""")
+
     high_dpi = param.Boolean(default=True, doc="""
         Whether to optimize output for high-dpi displays.""")
 
@@ -217,10 +227,13 @@ class Matplotlib(PNG, IPyWidget):
         subplots and other artist elements.""")
 
     _rename: ClassVar[Mapping[str, str | None]] = {
-        'object': 'text', 'interactive': None, 'dpi': None,  'tight': None, 'high_dpi': None
+        'object': 'text', 'interactive': None, 'dpi': None,  'tight': None,
+        'high_dpi': None, 'format': None, 'encode': None
     }
 
-    _rerender_params = PNG._rerender_params + ['object', 'dpi', 'tight']
+    _rerender_params = PNG._rerender_params + [
+        'interactive', 'object', 'dpi', 'tight', 'high_dpi'
+    ]
 
     @classmethod
     def applies(cls, obj: Any) -> float | bool | None:
@@ -236,8 +249,6 @@ class Matplotlib(PNG, IPyWidget):
     def __init__(self, object=None, **params):
         super().__init__(object, **params)
         self._managers = {}
-        self._explicit_width = params.get('width') is not None
-        self._explicit_height = params.get('height') is not None
 
     def _get_widget(self, fig):
         import matplotlib.backends
@@ -261,60 +272,54 @@ class Matplotlib(PNG, IPyWidget):
         canvas.mpl_connect('close_event', closer)
         return manager
 
-    @param.depends('width', watch=True)
-    def _set_explicict_width(self):
-        self._explicit_width = self.width is not None
+    @property
+    def _img_type(self):
+        if self.format == 'png':
+            return PNG
+        elif self.format == 'svg':
+            return SVG
+        else:
+            return PDF
 
-    @param.depends('height', watch=True)
-    def _set_explicict_height(self):
-        self._explicit_height = self.height is not None
+    @property
+    def filetype(self):
+        return self._img_type.filetype
 
-    def _update_dimensions(self):
-        w, h = self.object.get_size_inches()
-        dpi = self.dpi / 2. if self.high_dpi else self.dpi
-        with param.discard_events(self):
-            if not self._explicit_width:
-                if self._explicit_height:
-                    self.width = int(self.height * (w/h))
-                else:
-                    self.width = int(dpi * w)
-                self._explicit_width = False
-            if not self._explicit_height:
-                if self._explicit_width:
-                    self.height = int(self.width * (w/h))
-                else:
-                    self.height = self.height or int(dpi * h)
-                self._explicit_height = False
+    def _transform_object(self, obj: Any) -> Dict[str, Any]:
+        return self._img_type._transform_object(self, obj)
+
+    def _imgshape(self, data):
+        try:
+            return self._img_type._imgshape(data)
+        except TypeError:
+            return self._img_type._imgshape(self, data)
+
+    def _format_html(
+        self, src: str, width: str | None = None, height: str | None = None
+    ):
+        return self._img_type._format_html(self, src, width, height)
 
     def _get_model(
         self, doc: Document, root: Optional[Model] = None,
         parent: Optional[Model] = None, comm: Optional[Comm] = None
     ) -> Model:
-        self._update_dimensions()
         if not self.interactive:
-            model = PNG._get_model(self, doc, root, parent, comm)
-            return model
+            return self._img_type._get_model(self, doc, root, parent, comm)
         self.object.set_dpi(self.dpi)
         manager = self._get_widget(self.object)
-        props = self._process_param_change(self._init_params())
-        kwargs = {k: v for k, v in props.items()
-                  if k not in self._rerender_params+['interactive']}
-        kwargs['width'] = self.width
-        kwargs['height'] = self.height
-        kwargs['sizing_mode'] = self.sizing_mode
+        properties = self._get_properties(doc)
+        del properties['text']
         model = self._get_ipywidget(
-            manager.canvas, doc, root, comm, **kwargs
+            manager.canvas, doc, root, comm, **properties
         )
-        if root is None:
-            root = model
+        root = root or model
         self._models[root.ref['id']] = (model, parent)
         self._managers[root.ref['id']] = manager
         return model
 
     def _update(self, ref: str, model: Model) -> None:
         if not self.interactive:
-            self._update_dimensions()
-            model.update(**self._get_properties())
+            model.update(**self._get_properties(model.document))
             return
         manager = self._managers[ref]
         if self.object is not manager.canvas.figure:
@@ -327,9 +332,9 @@ class Matplotlib(PNG, IPyWidget):
             manager.canvas.handle_resize(event)
         manager.canvas.draw_idle()
 
-    def _data(self):
+    def _data(self, obj):
         try:
-            self.object.set_dpi(self.dpi)
+            obj.set_dpi(self.dpi)
         except Exception as ex:
             raise Exception("The Matplotlib backend is not configured. Try adding `matplotlib.use('agg')`") from ex
         b = BytesIO()
@@ -339,7 +344,14 @@ class Matplotlib(PNG, IPyWidget):
         else:
             bbox_inches = None
 
-        self.object.canvas.print_figure(b, bbox_inches=bbox_inches)
+        obj.canvas.print_figure(
+            b,
+            format=self.format,
+            facecolor=obj.get_facecolor(),
+            edgecolor=obj.get_edgecolor(),
+            dpi=self.dpi,
+            bbox_inches=bbox_inches
+        )
         return b.getvalue()
 
 
@@ -357,17 +369,19 @@ class RGGPlot(PNG):
 
     _rerender_params = PNG._rerender_params + ['object', 'dpi', 'width', 'height']
 
+    _rename: ClassVar[Mapping[str, str | None]] = {'dpi': None}
+
     @classmethod
     def applies(cls, obj: Any) -> float | bool | None:
         return type(obj).__name__ == 'GGPlot' and hasattr(obj, 'r_repr')
 
-    def _data(self):
+    def _data(self, obj):
         from rpy2 import robjects
         from rpy2.robjects.lib import grdevices
         with grdevices.render_to_bytesio(grdevices.png,
                  type="cairo-png", width=self.width, height=self.height,
                  res=self.dpi, antialias="subpixel") as b:
-            robjects.r("print")(self.object)
+            robjects.r("print")(obj)
         return b.getvalue()
 
 
@@ -387,25 +401,6 @@ class YT(HTML):
                 hasattr(obj, "plots") and
                 hasattr(obj, "_repr_html_"))
 
-    def _get_properties(self):
-        p = super()._get_properties()
-        if self.object is None:
-            return p
-
-        width = height = 0
-        if self.width  is None or self.height is None:
-            for k,v in self.object.plots.items():
-                if hasattr(v, "_repr_png_"):
-                    img = v._repr_png_()
-                    w,h = PNG._imgshape(img)
-                    height += h
-                    width = max(w, width)
-
-        if self.width  is None: p["width"]  = width
-        if self.height is None: p["height"] = height
-
-        return p
-
 
 class Folium(HTML):
     """
@@ -423,9 +418,8 @@ class Folium(HTML):
         return (getattr(obj, '__module__', '').startswith('folium.') and
                 hasattr(obj, "_repr_html_"))
 
-    def _get_properties(self):
-        properties = super()._get_properties()
-        text = '' if self.object is None else self.object
+    def _transform_object(self, obj: Any) -> Dict[str, Any]:
+        text = '' if obj is None else obj
         if hasattr(text, '_repr_html_'):
             text = text._repr_html_().replace(FOLIUM_BEFORE, FOLIUM_AFTER)
-        return dict(properties, text=escape(text))
+        return dict(object=escape(text))
