@@ -10,7 +10,7 @@ import zipfile
 
 from abc import abstractmethod
 from typing import (
-    TYPE_CHECKING, Any, ClassVar, Dict, Mapping, Optional,
+    IO, TYPE_CHECKING, Any, ClassVar, Dict, Mapping, Optional,
 )
 from urllib.request import urlopen
 
@@ -48,7 +48,7 @@ class AbstractVTK(PaneBase):
               axis' ticks.
           - ``labels`` (array of strings) - optional.
               Label displayed respectively to the `ticks` positions.
-              If `labels` are not defined they are infered from the
+              If `labels` are not defined they are inferred from the
               `ticks` array.
           - ``digits``: number of decimal digits when `ticks` are converted to `labels`.
           - ``fontsize``: size in pts of the ticks labels.
@@ -344,9 +344,10 @@ class BaseVTKRenderWindow(AbstractVTK):
         ren_win.Render()
         scene = rws.serializeInstance(None, ren_win, context.getReferenceId(ren_win), context, 0)
         scene['properties']['numberOfLayers'] = 2 #On js side the second layer is for the orientation widget
-        arrays = {name: context.getCachedDataArray(name, binary=binary, compression=compression)
-                    for name in context.dataArrayCache.keys()
-                    if name not in exclude_arrays}
+        arrays = {
+            name: context.getCachedDataArray(name, binary=True, compression=False)
+            for name in context.dataArrayCache.keys() if name not in exclude_arrays
+        }
         annotations = context.getAnnotations()
         return scene, arrays, annotations
 
@@ -389,16 +390,14 @@ class VTKRenderWindow(BaseVTKRenderWindow):
         VTKSynchronizedPlot = lazy_load(
             'panel.models.vtk', 'VTKSynchronizedPlot', isinstance(comm, JupyterComm), root
         )
-        props = self._process_param_change(self._init_params())
+        props = self._get_properties(doc)
         if self.object is not None:
             props.update(scene=self._scene, arrays=self._arrays, color_mappers=self.color_mappers)
         model = VTKSynchronizedPlot(**props)
-
-        if root is None:
-            root = model
-        self._link_props(model,
-                         ['enable_keybindings', 'orientation_widget'],
-                         doc, root, comm)
+        root = root or model
+        self._link_props(
+            model, ['enable_keybindings', 'orientation_widget'], doc, root, comm
+        )
         self._models[root.ref['id']] = (model, parent)
         return model
 
@@ -460,16 +459,12 @@ class VTKRenderWindowSynchronized(BaseVTKRenderWindow, SyncHelpers):
         )
         scene, arrays, annotations = self._serialize_ren_win(self.object, context)
         self._update_color_mappers()
-        props = self._process_param_change(self._init_params())
+        props = self._get_properties(doc)
         props.update(scene=scene, arrays=arrays, annotations=annotations, color_mappers=self.color_mappers)
         model = VTKSynchronizedPlot(**props)
-
-        if root is None:
-            root = model
-        self._link_props(model,
-                         ['camera', 'color_mappers', 'enable_keybindings', 'one_time_reset',
-                          'orientation_widget'],
-                         doc, root, comm)
+        root = root or model
+        linked = ['camera', 'color_mappers', 'enable_keybindings', 'one_time_reset', 'orientation_widget']
+        self._link_props(model, linked, doc, root, comm)
         self._contexts[model.id] =  context
         self._models[root.ref['id']] = (model, parent)
         return model
@@ -520,7 +515,8 @@ class VTKRenderWindowSynchronized(BaseVTKRenderWindow, SyncHelpers):
             'physicalTranslation',
             'physicalScale',
             'physicalViewUp',
-            'physicalViewNorth'
+            'physicalViewNorth',
+            'remoteId',
         ]
         if self.camera is not None:
             for k, v in self.camera.items():
@@ -570,7 +566,7 @@ class VTKVolume(AbstractVTK):
         light direction and the object surface normal.""")
 
     display_volume = param.Boolean(default=True, doc="""
-        If set to True, the 3D respresentation of the volume is
+        If set to True, the 3D representation of the volume is
         displayed using ray casting.""")
 
     display_slices = param.Boolean(default=False, doc="""
@@ -594,7 +590,7 @@ class VTKVolume(AbstractVTK):
     mapper = param.Dict(doc="Lookup Table in format {low, high, palette}")
 
     max_data_size = param.Number(default=(256 ** 3) * 2 / 1e6, doc="""
-        Maximum data size transfert allowed without subsampling""")
+        Maximum data size transfer allowed without subsampling""")
 
     nan_opacity = param.Number(default=1., bounds=(0., 1.), doc="""
         Opacity applied to nan values in slices""")
@@ -606,7 +602,7 @@ class VTKVolume(AbstractVTK):
         The value must be specified as an hexadecimal color string.""")
 
     rescale = param.Boolean(default=False, doc="""
-        If set to True the colormap is rescaled beween min and max
+        If set to True the colormap is rescaled between min and max
         value of the non-transparent pixel, otherwise  the full range
         of the pixel values are used.""")
 
@@ -735,19 +731,21 @@ class VTKVolume(AbstractVTK):
     @classmethod
     def register_serializer(cls, class_type, serializer):
         """
-        Register a seriliazer for a given type of class.
+        Register a serializer for a given type of class.
         A serializer is a function which take an instance of `class_type`
         (like a vtk.vtkImageData) as input and return a numpy array of the data
         """
         cls._serializers.update({class_type:serializer})
 
     def _volume_from_array(self, sub_array):
-        return dict(buffer=base64encode(sub_array.ravel(order='F')),
-                    dims=sub_array.shape,
-                    spacing=self._sub_spacing,
-                    origin=self.origin,
-                    data_range=(np.nanmin(sub_array), np.nanmax(sub_array)),
-                    dtype=sub_array.dtype.name)
+        return dict(
+            buffer=base64encode(sub_array.ravel(order='F')),
+            dims=sub_array.shape,
+            spacing=self._sub_spacing,
+            origin=self.origin,
+            data_range=(np.nanmin(sub_array), np.nanmax(sub_array)),
+            dtype=sub_array.dtype.name
+        )
 
     def _get_volume_data(self):
         if self.object is None:
@@ -825,7 +823,6 @@ class VTKJS(AbstractVTK):
 
     _updates = True
 
-
     def __init__(self, object=None, **params):
         super().__init__(object, **params)
         self._vtkjs = None
@@ -843,13 +840,12 @@ class VTKJS(AbstractVTK):
         Should return the bokeh model to be rendered.
         """
         VTKJSPlot = lazy_load('panel.models.vtk', 'VTKJSPlot', isinstance(comm, JupyterComm), root)
-        props = self._process_param_change(self._init_params())
+        props = self._get_properties(doc)
         vtkjs = self._get_vtkjs()
         if vtkjs is not None:
             props['data'] = base64encode(vtkjs)
         model = VTKJSPlot(**props)
-        if root is None:
-            root = model
+        root = root or model
         self._link_props(model, ['camera', 'enable_keybindings', 'orientation_widget'], doc, root, comm)
         self._models[root.ref['id']] = (model, parent)
         return model
@@ -873,6 +869,16 @@ class VTKJS(AbstractVTK):
         vtkjs = self._get_vtkjs()
         model.data = base64encode(vtkjs) if vtkjs is not None else vtkjs
 
-    def export_vtkjs(self, filename='vtk_panel.vtkjs'):
-        with open(filename, 'wb') as f:
-            f.write(self._get_vtkjs())
+    def export_vtkjs(self, filename: str | IO ='vtk_panel.vtkjs'):
+        """
+        Exports current VTK data to .vtkjs file.
+
+        Arguments
+        ---------
+        filename: str | IO
+        """
+        if hasattr(filename, 'write'):
+            filename.write(self._get_vtkjs())
+        else:
+            with open(filename, 'wb') as f:
+                f.write(self._get_vtkjs())
