@@ -658,7 +658,7 @@ class Reactive(Syncable, Viewable):
     def _update_properties(self, *events: param.parameterized.Event, doc: Document) -> Dict[str, Any]:
         params, _ = self._design.params(self, doc) if self._design else ({}, None)
         changes = {event.name: event.new for event in events}
-        if 'stylesheets' in changes and 'stylsheets' in params:
+        if 'stylesheets' in changes and 'stylesheets' in params:
             changes['stylesheets'] = params['stylesheets'] + changes['stylesheets']
         return self._process_param_change(changes)
 
