@@ -11,22 +11,25 @@ import uuid
 from functools import partial
 from pathlib import Path, PurePath
 from typing import (
-    IO, TYPE_CHECKING, Any, ClassVar, Dict, List, Optional, Tuple, Type,
+    IO, TYPE_CHECKING, Any, ClassVar, Dict, List, Literal, Optional, Tuple,
+    Type,
 )
 
 import param
 
 from bokeh.document.document import Document
+from bokeh.models import LayoutDOM
 from bokeh.settings import settings as _settings
 from pyviz_comms import JupyterCommManager as _JupyterCommManager
 
 from ..config import _base_config, config, panel_extension
+from ..io.document import init_doc
 from ..io.model import add_to_doc
 from ..io.notebook import render_template
 from ..io.notifications import NotificationArea
 from ..io.resources import (
-    BUNDLE_DIR, CDN_DIST, CSS_URLS, DOC_DIST, JS_URLS, LOCAL_DIST, _env,
-    component_resource_path, resolve_custom_path,
+    BUNDLE_DIR, CDN_DIST, ResourceComponent, _env, component_resource_path,
+    get_dist_path, loading_css, resolve_custom_path,
 )
 from ..io.save import save
 from ..io.state import curdoc_locked, state
@@ -37,27 +40,24 @@ from ..pane import (
 )
 from ..pane.image import ImageBase
 from ..reactive import ReactiveHTML
-from ..util import isurl, url_path
-from ..viewable import Renderable, ServableMixin, Viewable
+from ..theme.base import (
+    THEMES, DefaultTheme, Design, Theme,
+)
+from ..util import isurl
+from ..viewable import (
+    MimeRenderMixin, Renderable, ServableMixin, Viewable,
+)
 from ..widgets import Button
 from ..widgets.indicators import BooleanIndicator, LoadingSpinner
-from .theme import THEMES, DefaultTheme, Theme
 
 if TYPE_CHECKING:
     from bokeh.model import Model
     from bokeh.server.contexts import BokehSessionContext
     from jinja2 import Template as _Template
     from pyviz_comms import Comm
-    from typing_extensions import Literal, TypedDict
 
     from ..io.location import Location
-
-    class ResourcesType(TypedDict):
-        css: Dict[str, str]
-        js:  Dict[str, str]
-        js_modules: Dict[str, str]
-        extra_css: List[str]
-        raw_css: List[str]
+    from ..io.resources import ResourcesType
 
 
 _server_info: str = (
@@ -68,7 +68,11 @@ _server_info: str = (
 FAVICON_URL: str = "/static/extensions/panel/images/favicon.ico"
 
 
-class BaseTemplate(param.Parameterized, ServableMixin):
+class BaseTemplate(param.Parameterized, MimeRenderMixin, ServableMixin):
+
+    design = param.ClassSelector(class_=Design, default=Design,
+                                 is_instance=False, instantiate=False, doc="""
+        A Design applies themes to a template.""")
 
     location = param.Boolean(default=False, doc="""
         Whether to add a Location component to this Template.
@@ -76,8 +80,11 @@ class BaseTemplate(param.Parameterized, ServableMixin):
         either insert all available roots or explicitly embed
         the location root with : {{ embed(roots.location) }}.""")
 
+    theme = param.ClassSelector(class_=Theme, default=DefaultTheme,
+                                constant=True, is_instance=False, instantiate=False)
+
     # Dictionary of property overrides by Viewable type
-    _modifiers: ClassVar[Dict[Type[Viewable], Dict[str, Any]]] = {}
+    modifiers: ClassVar[Dict[Type[Viewable], Dict[str, Any]]] = {}
 
     __abstract = True
 
@@ -101,6 +108,11 @@ class BaseTemplate(param.Parameterized, ServableMixin):
         self._documents: List[Document] = []
         self._server = None
         self._layout = self._build_layout()
+        self._setup_design()
+
+    @param.depends('design', watch=True)
+    def _setup_design(self):
+        self._design = self.design(theme=self.theme)
 
     def _build_layout(self) -> Column:
         str_repr = Str(repr(self))
@@ -128,35 +140,6 @@ class BaseTemplate(param.Parameterized, ServableMixin):
         ])
         return f'{type(self).__name__}{spacer}{objs}'
 
-    def _apply_hooks(self, viewable: Viewable, root: Model) -> None:
-        ref = root.ref['id']
-        for o in viewable.select():
-            self._apply_modifiers(o, ref)
-
-    @classmethod
-    def _apply_modifiers(cls, viewable: Viewable, mref: str) -> None:
-        if mref not in viewable._models:
-            return
-        model, _ = viewable._models[mref]
-        modifiers = cls._modifiers.get(type(viewable), {})
-        child_modifiers = modifiers.get('children', {})
-        if child_modifiers:
-            for child in viewable:
-                child_params = {
-                    k: v for k, v in child_modifiers.items()
-                    if getattr(child, k) == child.param[k].default
-                }
-                child.param.set_param(**child_params)
-                child_props = child._process_param_change(child_params)
-                child._models[mref][0].update(**child_props)
-        params = {
-            k: v for k, v in modifiers.items() if k != 'children' and
-            getattr(viewable, k) == viewable.param[k].default
-        }
-        viewable.param.update(**params)
-        props = viewable._process_param_change(params)
-        model.update(**props)
-
     def _apply_root(self, name: str, model: Model, tags: List[str]) -> None:
         pass
 
@@ -171,7 +154,9 @@ class BaseTemplate(param.Parameterized, ServableMixin):
         title: Optional[str] = None, notebook: bool = False,
         location: bool | Location=True
     ):
+        # Initialize document
         document: Document = doc or curdoc_locked()
+        document = init_doc(document)
         self._documents.append(document)
         if document not in state._templates:
             state._templates[document] = self
@@ -188,32 +173,45 @@ class BaseTemplate(param.Parameterized, ServableMixin):
         # which assume that all models are owned by a single root can
         # link objects across multiple roots in a template.
         col = Column()
-        preprocess_root = col.get_root(document, comm)
-        col._hooks.append(self._apply_hooks)
+        preprocess_root = col.get_root(document, comm, preprocess=False)
+        col._hooks.append(self._design._apply_hooks)
         ref = preprocess_root.ref['id']
-        objs, models = [], []
 
+        # Add all render items to the document
+        objs, models = [], []
+        sizing_modes = {}
         for name, (obj, tags) in self._render_items.items():
-            if self._apply_hooks not in obj._hooks:
-                obj._hooks.append(self._apply_hooks)
-            # We skip preprocessing on the individual roots
+
+            # Render root without pre-processing
             model = obj.get_root(document, comm, preprocess=False)
-            mref = model.ref['id']
-            document.on_session_destroyed(obj._server_destroy) # type: ignore
-            for sub in obj.select(Viewable):
-                submodel = sub._models.get(mref)
-                if submodel is None:
-                    continue
-                sub._models[ref] = submodel
-                if isinstance(sub, HoloViews) and mref in sub._plots:
-                    sub._plots[ref] = sub._plots.get(mref)
-            obj._documents[document] = model
             model.name = name
-            model.tags = tags
-            self._apply_root(name, model, tags)
+            model.tags = model.tags + [tag for tag in tags if tag not in model.tags]
+            mref = model.ref['id']
+            if isinstance(model, LayoutDOM):
+                sizing_modes[mref] = model.sizing_mode
+                if self._design._apply_hooks not in obj._hooks:
+                    obj._hooks.append(self._design._apply_hooks)
+
+                # Alias model ref with the fake root ref to ensure that
+                # pre-processor correctly operates on fake root
+                for sub in obj.select(Viewable):
+                    submodel = sub._models.get(mref)
+                    if submodel is None:
+                        continue
+                    sub._models[ref] = submodel
+                    if isinstance(sub, HoloViews) and mref in sub._plots:
+                        sub._plots[ref] = sub._plots.get(mref)
+
+                # Apply any template specific hooks to root
+                self._apply_root(name, model, tags)
+
+                # Add document
+                obj._documents[document] = model
+                objs.append(obj)
+                models.append(model)
+
             add_to_doc(model, document, hold=bool(comm))
-            objs.append(obj)
-            models.append(model)
+            document.on_session_destroyed(obj._server_destroy) # type: ignore
 
         # Here we ensure that the preprocessor is run across all roots
         # and set up session cleanup hooks for the fake root.
@@ -221,14 +219,19 @@ class BaseTemplate(param.Parameterized, ServableMixin):
         state._views[ref] = (col, preprocess_root, document, comm)
         col.objects = objs
         preprocess_root.children[:] = models
+        preprocess_root.document = document
+        self._design.apply(col, preprocess_root, isolated=False)
         col._preprocess(preprocess_root)
         col._documents[document] = preprocess_root
         document.on_session_destroyed(col._server_destroy) # type: ignore
 
+        # Apply the jinja2 template and update template variables
         if notebook:
             document.template = self.nb_template
         else:
             document.template = self.template
+
+        document._template_variables['sizing_modes'] = sizing_modes
         document._template_variables.update(self._render_variables)
         return document
 
@@ -347,6 +350,43 @@ class BaseTemplate(param.Parameterized, ServableMixin):
         """
         return self._init_doc(doc, title=title, location=location)
 
+    def servable(
+        self, title: Optional[str] = None, location: bool | 'Location' = True,
+        area: str = 'main', target: Optional[str] = None
+    ) -> 'BaseTemplate':
+        """
+        Serves the template and returns self to allow it to display
+        itself in a notebook context.
+
+        Arguments
+        ---------
+        title : str
+          A string title to give the Document (if served as an app)
+        location : boolean or panel.io.location.Location
+          Whether to create a Location component to observe and
+          set the URL location.
+        area: str (deprecated)
+          The area of a template to add the component too. Only has an
+          effect if pn.config.template has been set.
+        target: str
+          Target area to write to. If a template has been configured
+          on pn.config.template this refers to the target area in the
+          template while in pyodide this refers to the ID of the DOM
+          node to write to.
+
+        Returns
+        -------
+        The template
+        """
+        if curdoc_locked().session_context and config.template:
+            raise RuntimeError(
+                'Cannot mark template as servable if a global template '
+                'is defined. Either explicitly construct a template and '
+                'serve it OR set `pn.config.template`, whether directly '
+                'or via `pn.extension(template=...)`, not both.'
+            )
+        return super().servable(title, location, area, target)
+
     def select(self, selector=None):
         """
         Iterates over the Template and any potential children in the
@@ -379,8 +419,6 @@ class TemplateActions(ReactiveHTML):
 
     close_modal = param.Integer(default=0)
 
-    margin = param.Integer(default=0)
-
     _template: ClassVar[str] = ""
 
     _scripts: ClassVar[Dict[str, List[str] | str]] = {
@@ -389,7 +427,7 @@ class TemplateActions(ReactiveHTML):
     }
 
 
-class BasicTemplate(BaseTemplate):
+class BasicTemplate(BaseTemplate, ResourceComponent):
     """
     BasicTemplate provides a baseclass for templates with a basic
     organization including a header, sidebar and main area. Unlike the
@@ -421,7 +459,7 @@ class BasicTemplate(BaseTemplate):
     sidebar = param.ClassSelector(class_=ListLike, constant=True, doc="""
         A list-like container which populates the sidebar.""")
 
-    sidebar_width = param.Integer(330, doc="""
+    sidebar_width = param.Integer(default=330, doc="""
         The width of the sidebar in pixels. Default is 330.""")
 
     modal = param.ClassSelector(class_=ListLike, constant=True, doc="""
@@ -488,9 +526,6 @@ class BasicTemplate(BaseTemplate):
     header_color = param.String(doc="""
         Optional header text color override.""")
 
-    theme = param.ClassSelector(class_=Theme, default=DefaultTheme,
-                                constant=True, is_instance=False, instantiate=False)
-
     location = param.Boolean(default=True, readonly=True)
 
     _actions = param.ClassSelector(default=TemplateActions(), class_=TemplateActions)
@@ -499,10 +534,6 @@ class BasicTemplate(BaseTemplate):
     # Resources #
     #############
 
-    # Resource locations for bundled resources
-    _CDN: ClassVar[str] = CDN_DIST
-    _LOCAL: ClassVar[str] = LOCAL_DIST
-
     # pathlib.Path pointing to local CSS file(s)
     _css: ClassVar[Path | str | List[Path | str] | None] = None
 
@@ -517,12 +548,10 @@ class BasicTemplate(BaseTemplate):
         'css': {}, 'js': {}, 'js_modules': {}, 'tarball': {}
     }
 
-    _modifiers: ClassVar[Dict[Type[Viewable], Dict[str, Any]]] = {}
-
     __abstract = True
 
     def __init__(self, **params):
-        template = self._template.read_text()
+        template = self._template.read_text(encoding='utf-8')
         if 'header' not in params:
             params['header'] = ListLike()
         else:
@@ -539,8 +568,11 @@ class BasicTemplate(BaseTemplate):
             params['modal'] = ListLike()
         else:
             params['modal'] = self._get_params(params['modal'], self.param.modal.class_)
-        if 'theme' in params and isinstance(params['theme'], str):
-            params['theme'] = THEMES[params['theme']]
+        if 'theme' in params:
+            if isinstance(params['theme'], str):
+                params['theme'] = THEMES[params['theme']]
+        else:
+            params['theme'] = THEMES[config.theme]
         if 'favicon' in params and isinstance(params['favicon'], PurePath):
             params['favicon'] = str(params['favicon'])
         if 'notifications' not in params and config.notifications:
@@ -554,6 +586,9 @@ class BasicTemplate(BaseTemplate):
         if 'embed(roots.notifications)' in template and self.notifications:
             self._render_items['notifications'] = (self.notifications, [])
             self._render_variables['notifications'] = True
+        if config.browser_info and 'embed(roots.browser_info)' in template and state.browser_info:
+            self._render_items['browser_info'] = (state.browser_info, [])
+            self._render_variables['browser_info'] = True
         self._update_busy()
         self.main.param.watch(self._update_render_items, ['objects'])
         self.modal.param.watch(self._update_render_items, ['objects'])
@@ -575,96 +610,57 @@ class BasicTemplate(BaseTemplate):
         document = super()._init_doc(doc, comm, title, notebook, location)
         if self.notifications:
             state._notifications[document] = self.notifications
-        if self.theme:
-            theme = self._get_theme()
-            if theme and theme.bokeh_theme:
-                document.theme = theme.bokeh_theme
+        if self._design.theme.bokeh_theme:
+            document.theme = self._design.theme.bokeh_theme
         return document
 
-    def _apply_hooks(self, viewable: Viewable, root: Model):
-        super()._apply_hooks(viewable, root)
-        theme = self._get_theme()
-        if theme and theme.bokeh_theme and root.document:
-            root.document.theme = theme.bokeh_theme
-        return
+    def resolve_resources(self, cdn: bool | Literal['auto'] = 'auto') -> ResourcesType:
+        """
+        Resolves the resources required for this template component.
 
-    def _get_theme(self) -> Theme | None:
-        for cls in type(self).__mro__:
-            try:
-                return self.theme.find_theme(cls)()
-            except Exception:
-                pass
-        return None
+        Arguments
+        ---------
+        cdn: bool | Literal['auto']
+            Whether to load resources from CDN or local server. If set
+            to 'auto' value will be automatically determine based on
+            global settings.
 
-    def _template_resources(self) -> ResourcesType:
-        clsname = type(self).__name__
+        Returns
+        -------
+        Dictionary containing JS and CSS resources.
+        """
+        cls = type(self)
+        resource_types = super().resolve_resources(cdn=cdn)
+        js_files = resource_types['js']
+        js_modules = resource_types['js_modules']
+        css_files = resource_types['css']
+        raw_css = resource_types['raw_css']
+
+        clsname = cls.__name__
         name = clsname.lower()
-        use_cdn = _settings.resources(default="server") != 'server'
-        if use_cdn:
-            dist_path = self._CDN
-        elif state.rel_path:
-            dist_path = f'{state.rel_path}/{self._LOCAL}'
-        else:
-            dist_path = f'{self._LOCAL}'
+        dist_path = get_dist_path(cdn=cdn)
 
-        # External resources
-        css_files: Dict[str, str] = {}
-        js_files: Dict[str, str] = {}
-        js_modules: Dict[str, str] = {}
-        resource_types: ResourcesType = {
-            'css': css_files,
-            'js': js_files,
-            'js_modules': js_modules,
-            'extra_css': [],
-            'raw_css': list(self.config.raw_css)
-        }
+        raw_css.extend(list(self.config.raw_css) + [loading_css()])
+        for rname, res in self._design.resolve_resources(cdn).items():
+            if isinstance(res, dict):
+                resource_types[rname].update(res)
+            else:
+                resource_types[rname] += [
+                    r for r in res if res not in resource_types[rname]
+                ]
 
-        resolved_resources: List[Literal['css', 'js', 'js_modules']] = ['css', 'js', 'js_modules']
-        for resource_type in resolved_resources:
-            if resource_type not in self._resources:
-                continue
-            resource_files = resource_types[resource_type]
-            shared = list((CSS_URLS if resource_type == 'css' else JS_URLS).values())
-            for rname, resource in self._resources[resource_type].items():
-                if resource.startswith(CDN_DIST):
-                    resource_path = resource.replace(f'{CDN_DIST}bundled/', '')
-                elif resource.startswith(config.npm_cdn):
-                    resource_path = resource.replace(config.npm_cdn, '')[1:]
-                else:
-                    resource_path = url_path(resource)
-                if resource in shared:
-                    prefixed = resource_path
-                else:
-                    rtype = 'css' if resource_type == 'css' else 'js'
-                    prefixed = f'{rtype}/{resource_path}'
-                if resource_type == 'js_modules' and not (state.rel_path or use_cdn):
-                    prefixed_dist = f'./{dist_path}'
-                else:
-                    prefixed_dist = dist_path
-                bundlepath = BUNDLE_DIR / prefixed.replace('/', os.path.sep)
-                if bundlepath:
-                    resource_files[rname] = f'{prefixed_dist}bundled/{prefixed}'
-                elif isurl(resource):
-                    resource_files[rname] = resource
-                elif resolve_custom_path(self, resource):
-                    resource_files[rname] = component_resource_path(
-                        self, f'_resources/{resource_type}', resource
-                    )
-
-        for name, js in self.config.js_files.items():
+        for rname, js in self.config.js_files.items():
             if '//' not in js and state.rel_path:
                 js = f'{state.rel_path}/{js}'
-            js_files[name] = js
-        for name, js in self.config.js_modules.items():
+            js_files[rname] = js
+        for rname, js in self.config.js_modules.items():
             if '//' not in js and state.rel_path:
                 js = f'{state.rel_path}/{js}'
-            js_modules[name] = js
-
-        extra_css = resource_types['extra_css']
-        for css in list(self.config.css_files):
+            js_modules[rname] = js
+        for i, css in enumerate(list(self.config.css_files)):
             if '//' not in css and state.rel_path:
                 css = f'{state.rel_path}/{css}'
-            extra_css.append(css)
+            css_files[f'config_{i}'] = css
 
         # CSS files
         base_css = self._css
@@ -672,14 +668,14 @@ class BasicTemplate(BaseTemplate):
             base_css = [base_css] if base_css else []
         for css in base_css:
             tmpl_name = name
-            for cls in type(self).__mro__[1:-5]:
-                if not issubclass(cls, BasicTemplate):
+            for scls in cls.__mro__[1:]:
+                if not issubclass(scls, BasicTemplate):
                     continue
-                elif cls._css is None:
+                elif scls._css is None:
                     break
-                tmpl_css = cls._css if isinstance(cls._css, list) else [cls._css]
+                tmpl_css = scls._css if isinstance(scls._css, list) else [scls._css]
                 if css in tmpl_css:
-                    tmpl_name = cls.__name__.lower()
+                    tmpl_name = scls.__name__.lower()
 
             css_file = os.path.basename(css)
             if (BUNDLE_DIR / tmpl_name / css_file).is_file():
@@ -711,27 +707,6 @@ class BasicTemplate(BaseTemplate):
             elif resolve_custom_path(self, js):
                 js_files[f'base_{js_name}'] = component_resource_path(self, '_js', js)
 
-        theme = self._get_theme()
-        if not theme:
-            return resource_types
-        if theme.base_css:
-            basename = os.path.basename(theme.base_css)
-            owner = type(theme).param.base_css.owner
-            owner_name = owner.__name__.lower()
-            if (BUNDLE_DIR / owner_name / basename).is_file():
-                css_files['theme_base'] = dist_path + f'bundled/{owner_name}/{basename}'
-            elif isurl(theme.base_css):
-                css_files['theme_base'] = theme.base_css
-            elif resolve_custom_path(theme, theme.base_css):
-                css_files['theme_base'] = component_resource_path(owner, 'base_css', theme.base_css)
-        if theme.css:
-            basename = os.path.basename(theme.css)
-            if (BUNDLE_DIR / name / basename).is_file():
-                css_files['theme'] = dist_path + f'bundled/{name}/{basename}'
-            elif isurl(theme.css):
-                css_files['theme'] = theme.css
-            elif resolve_custom_path(theme, theme.css):
-                css_files['theme'] = component_resource_path(theme, 'css', theme.css)
         return resource_types
 
     def _update_vars(self, *args) -> None:
@@ -751,20 +726,20 @@ class BasicTemplate(BaseTemplate):
             img = _panel(self.logo)
             if not isinstance(img, ImageBase):
                 raise ValueError(f"Could not determine file type of logo: {self.logo}.")
-            logo = img._b64()
+            logo = img._b64(img._data(img.object))
         else:
             logo = self.logo
         if os.path.isfile(self.favicon):
             img = _panel(self.favicon)
             if not isinstance(img, ImageBase):
                 raise ValueError(f"Could not determine file type of favicon: {self.favicon}.")
-            favicon = img._b64()
+            favicon = img._b64(img._data(img.object))
         else:
             if _settings.resources(default='server') == 'cdn' and self.favicon == FAVICON_URL:
-                favicon = DOC_DIST + "icons/favicon.ico"
+                favicon = CDN_DIST + "images/favicon.ico"
             else:
                 favicon = self.favicon
-        self._render_variables['template_resources'] = self._template_resources()
+        self._render_variables['template_resources'] = self.resolve_resources()
         self._render_variables['app_logo'] = logo
         self._render_variables['app_favicon'] = favicon
         self._render_variables['app_favicon_type'] = self._get_favicon_type(self.favicon)
@@ -772,6 +747,7 @@ class BasicTemplate(BaseTemplate):
         self._render_variables['header_color'] = self.header_color
         self._render_variables['main_max_width'] = self.main_max_width
         self._render_variables['sidebar_width'] = self.sidebar_width
+        self._render_variables['theme'] = self._design.theme
 
     def _update_busy(self) -> None:
         if self.busy_indicator:
@@ -799,15 +775,12 @@ class BasicTemplate(BaseTemplate):
                 del self._render_items[ref]
 
         new = event.new if isinstance(event.new, list) else event.new.values()
-        theme = self._get_theme()
-        if theme:
-            bk_theme = theme.bokeh_theme
+        if self._design.theme.bokeh_theme:
             for o in new:
                 if o in old:
                     continue
                 for hvpane in o.select(HoloViews):
-                    if bk_theme:
-                        hvpane.theme = bk_theme
+                    hvpane.theme = self._design.theme.bokeh_theme
 
         labels = {}
         for obj in new:
