@@ -1,4 +1,5 @@
 import datetime as dt
+import warnings
 
 from collections import OrderedDict
 
@@ -17,11 +18,11 @@ except Exception:
 plotly_available = pytest.mark.skipif(hv_plotly is None, reason="requires plotly backend")
 
 from bokeh.models import (
-    Column as BkColumn, ColumnDataSource, GlyphRenderer, GridBox, Line,
+    Column as BkColumn, ColumnDataSource, GlyphRenderer, GridPlot, Line,
     Row as BkRow, Scatter, Select as BkSelect, Slider as BkSlider,
     Spacer as BkSpacer,
 )
-from bokeh.plotting import Figure
+from bokeh.plotting import figure
 
 import panel as pn
 
@@ -29,6 +30,7 @@ from panel.depends import bind
 from panel.layout import Column, FlexBox, Row
 from panel.pane import HoloViews, PaneBase, panel
 from panel.tests.util import hv_available, mpl_available
+from panel.util.warnings import PanelDeprecationWarning
 from panel.widgets import (
     Checkbox, DiscreteSlider, FloatSlider, Select,
 )
@@ -85,7 +87,7 @@ def test_holoviews_pane_switch_backend(document, comm):
     # Replace Pane.object
     pane.backend = 'bokeh'
     model = row.children[0]
-    assert isinstance(model, Figure)
+    assert isinstance(model, figure)
 
     # Cleanup
     pane._cleanup(row)
@@ -103,7 +105,7 @@ def test_holoviews_pane_bokeh_renderer(document, comm):
     assert isinstance(row, BkRow)
     assert len(row.children) == 1
     model = row.children[0]
-    assert isinstance(model, Figure)
+    assert isinstance(model, figure)
     assert pane._models[row.ref['id']][0] is model
     renderers = [r for r in model.renderers if isinstance(r, GlyphRenderer)]
     assert len(renderers) == 1
@@ -113,7 +115,7 @@ def test_holoviews_pane_bokeh_renderer(document, comm):
     scatter = hv.Scatter([1, 2, 3])
     pane.object = scatter
     model = row.children[0]
-    assert isinstance(model, Figure)
+    assert isinstance(model, figure)
     renderers = [r for r in model.renderers if isinstance(r, GlyphRenderer)]
     assert len(renderers) == 1
     assert isinstance(renderers[0].glyph, Scatter)
@@ -138,7 +140,7 @@ def test_holoviews_pane_initialize_empty(document, comm):
 
     pane.object = hv.Curve([1, 2, 3])
     model = row.children[0]
-    assert isinstance(model, Figure)
+    assert isinstance(model, figure)
 
 @pytest.mark.usefixtures("hv_bokeh")
 @hv_available
@@ -154,7 +156,7 @@ def test_holoviews_pane_reflect_responsive(document, comm):
 
     pane.object = hv.Curve([1, 2, 3])
 
-    assert row.sizing_mode == 'fixed'
+    assert row.sizing_mode is None
     assert pane.sizing_mode == 'fixed'
 
 @pytest.mark.usefixtures("hv_bokeh")
@@ -166,7 +168,7 @@ def test_holoviews_pane_reflect_responsive_override(document, comm):
     # Create pane
     row = pane.get_root(document, comm=comm)
 
-    assert row.sizing_mode == 'fixed'
+    assert row.sizing_mode is None
     assert pane.sizing_mode == 'fixed'
 
     # Unset override
@@ -279,7 +281,7 @@ def test_holoviews_with_widgets(document, comm):
 
     hv_pane = HoloViews(hmap)
     layout = hv_pane.get_root(document, comm)
-    model = layout.children[0]
+    model = layout.children[0].children[0]
     assert len(hv_pane.widget_box.objects) == 2
     assert hv_pane.widget_box.objects[0].name == 'X'
     assert hv_pane.widget_box.objects[1].name == 'Y'
@@ -302,12 +304,12 @@ def test_holoviews_updates_widgets(document, comm):
 
     hv_pane.widgets = {'X': Select}
     assert isinstance(hv_pane.widget_box[0], Select)
-    assert isinstance(layout.children[1].children[0].children[0], BkSelect)
+    assert isinstance(layout.children[0].children[1].children[0], BkSelect)
 
     hv_pane.widgets = {'X': DiscreteSlider}
     assert isinstance(hv_pane.widget_box[0], DiscreteSlider)
-    assert isinstance(layout.children[1].children[0].children[0], BkColumn)
-    assert isinstance(layout.children[1].children[0].children[0].children[1], BkSlider)
+    assert isinstance(layout.children[0].children[1].children[0], BkColumn)
+    assert isinstance(layout.children[0].children[1].children[0].children[1], BkSlider)
 
 @hv_available
 def test_holoviews_widgets_update_plot(document, comm):
@@ -316,7 +318,7 @@ def test_holoviews_widgets_update_plot(document, comm):
     hv_pane = HoloViews(hmap, backend='bokeh')
     layout = hv_pane.get_root(document, comm)
 
-    cds = layout.children[0].select_one(ColumnDataSource)
+    cds = layout.children[0].select_one({'type': ColumnDataSource})
     assert cds.data['y'] == np.array([0])
     hv_pane.widget_box[0].value = 1
     hv_pane.widget_box[1].value = chr(65+1)
@@ -327,7 +329,7 @@ def test_holoviews_widgets_update_plot(document, comm):
 def test_holoviews_with_widgets_not_shown(document, comm):
     hmap = hv.HoloMap({(i, chr(65+i)): hv.Curve([i]) for i in range(3)}, kdims=['X', 'Y'])
 
-    hv_pane = HoloViews(hmap, show_widgets=False)
+    hv_pane = HoloViews(hmap)
     layout_obj = Column(hv_pane, hv_pane.widget_box)
     layout = layout_obj.get_root(document, comm)
     model = layout.children[0]
@@ -369,25 +371,23 @@ def test_holoviews_layouts(document, comm):
                     col = layout[1]
                     cmodel = model.children[1]
                     assert isinstance(col, Column)
-                    widgets, hv_col = col
-                    hv_obj = hv_col[1]
-                    wmodel, hv_model = cmodel.children[0],  cmodel.children[1].children[1]
+                    widgets, hv_obj = col
+                    wmodel, hv_model = cmodel.children[0],  cmodel.children[1]
                 elif loc.startswith('bottom'):
                     col = layout[1]
                     cmodel = model.children[1]
                     assert isinstance(col, Column)
-                    hv_col, widgets = col
-                    hv_obj = hv_col[1]
-                    wmodel, hv_model = cmodel.children[1],  cmodel.children[0].children[1]
+                    hv_obj, widgets = col
+                    wmodel, hv_model = cmodel.children[1],  cmodel.children[0]
             else:
                 if loc.startswith('left'):
-                    assert len(layout) == 2
-                    widgets, hv_obj = layout
-                    wmodel, hv_model = model.children
+                    assert len(layout) == 1
+                    widgets, hv_obj = layout[0]
+                    wmodel, hv_model = model.children[0].children
                 elif loc.startswith('right'):
-                    assert len(layout) == 2
-                    hv_obj, widgets = layout
-                    hv_model, wmodel = model.children
+                    assert len(layout) == 1
+                    hv_obj, widgets = layout[0]
+                    hv_model, wmodel = model.children[0].children
                 elif loc.startswith('top'):
                     assert len(layout) == 1
                     col = layout[0]
@@ -403,16 +403,10 @@ def test_holoviews_layouts(document, comm):
                     hv_obj, widgets = col
                     hv_model, wmodel = cmodel.children
             assert hv_pane is hv_obj
-            assert isinstance(hv_model, Figure)
+            assert isinstance(hv_model, figure)
 
-            if loc in ('left', 'right', 'top', 'bottom',
-                       'top_right', 'right_bottom', 'bottom_right',
-                       'left_bottom'):
-                box = widgets[1]
-                boxmodel = wmodel.children[1]
-            else:
-                box = widgets[0]
-                boxmodel = wmodel.children[0]
+            box = widgets
+            boxmodel = wmodel
             assert hv_pane.widget_box is box
             assert isinstance(boxmodel, BkColumn)
             assert isinstance(boxmodel.children[0], BkColumn)
@@ -491,7 +485,7 @@ def test_holoviews_linked_axes(document, comm):
 
     row_model = layout.get_root(document, comm=comm)
 
-    p1, p2 = row_model.select({'type': Figure})
+    p1, p2 = row_model.select({'type': figure})
 
     assert p1.x_range is p2.x_range
     assert p1.y_range is p2.y_range
@@ -506,7 +500,7 @@ def test_holoviews_linked_axes_flexbox(document, comm):
 
     row_model = layout.get_root(document, comm=comm)
 
-    p1, p2 = row_model.select({'type': Figure})
+    p1, p2 = row_model.select({'type': figure})
 
     assert p1.x_range is p2.x_range
     assert p1.y_range is p2.y_range
@@ -521,7 +515,7 @@ def test_holoviews_linked_axes_merged_ranges(document, comm):
 
     row_model = layout.get_root(document, comm=comm)
 
-    p1, p2 = row_model.select({'type': Figure})
+    p1, p2 = row_model.select({'type': figure})
 
     assert p1.x_range is p2.x_range
     assert p1.y_range is p2.y_range
@@ -538,7 +532,7 @@ def test_holoviews_linked_x_axis(document, comm):
 
     row_model = layout.get_root(document, comm=comm)
 
-    p1, p2 = row_model.select({'type': Figure})
+    p1, p2 = row_model.select({'type': figure})
 
     assert p1.x_range is p2.x_range
     assert p1.y_range is not p2.y_range
@@ -553,7 +547,7 @@ def test_holoviews_axiswise_not_linked_axes(document, comm):
 
     row_model = layout.get_root(document, comm=comm)
 
-    p1, p2 = row_model.select({'type': Figure})
+    p1, p2 = row_model.select({'type': figure})
 
     assert p1.x_range is not p2.x_range
     assert p1.y_range is not p2.y_range
@@ -568,7 +562,7 @@ def test_holoviews_shared_axes_opt_not_linked_axes(document, comm):
 
     row_model = layout.get_root(document, comm=comm)
 
-    p1, p2 = row_model.select({'type': Figure})
+    p1, p2 = row_model.select({'type': figure})
 
     assert p1.x_range is not p2.x_range
     assert p1.y_range is not p2.y_range
@@ -586,7 +580,7 @@ def test_holoviews_not_linked_axes(document, comm):
 
     row_model = layout.get_root(document, comm=comm)
 
-    p1, p2 = row_model.select({'type': Figure})
+    p1, p2 = row_model.select({'type': figure})
 
     assert p1.x_range is not p2.x_range
     assert p1.y_range is not p2.y_range
@@ -608,8 +602,8 @@ def test_holoviews_link_across_panes(document, comm):
     assert len(row.children) == 2
     p1, p2 = row.children
 
-    assert isinstance(p1, Figure)
-    assert isinstance(p2, Figure)
+    assert isinstance(p1, figure)
+    assert isinstance(p2, figure)
 
     range_tool = row.select_one({'type': RangeTool})
     assert isinstance(range_tool, RangeTool)
@@ -632,13 +626,13 @@ def test_holoviews_link_after_adding_item(document, comm):
     assert len(row.children) == 1
     p1, = row.children
 
-    assert isinstance(p1, Figure)
+    assert isinstance(p1, figure)
     range_tool = row.select_one({'type': RangeTool})
     assert range_tool is None
 
     layout.append(pn.panel(c2, backend='bokeh'))
     _, p2 = row.children
-    assert isinstance(p2, Figure)
+    assert isinstance(p2, figure)
     range_tool = row.select_one({'type': RangeTool})
     assert isinstance(range_tool, RangeTool)
     assert range_tool.x_range == p2.x_range
@@ -658,29 +652,40 @@ def test_holoviews_link_within_pane(document, comm):
     column = pane.get_root(document, comm=comm)
 
     assert len(column.children) == 1
-    subcolumn = column.children[0]
-    assert isinstance(subcolumn, BkColumn)
-    assert len(subcolumn.children) == 2
-    toolbar, subsubcolumn = subcolumn.children
-    assert isinstance(subsubcolumn, GridBox)
-    assert len(subsubcolumn.children) == 2
-    (p1, _, _), (p2, _, _) = subsubcolumn.children
+    grid_plot = column.children[0]
+    assert isinstance(grid_plot, GridPlot)
+    assert len(grid_plot.children) == 2
+    (p1, _, _), (p2, _, _) = grid_plot.children
 
-    assert isinstance(p1, Figure)
-    assert isinstance(p2, Figure)
+    assert isinstance(p1, figure)
+    assert isinstance(p2, figure)
 
-    range_tool = subsubcolumn.select_one({'type': RangeTool})
+    range_tool = grid_plot.select_one({'type': RangeTool})
     assert isinstance(range_tool, RangeTool)
     assert range_tool.x_range == p2.x_range
 
 
+@hv_available
+def test_holoviews_property_override_old_method(document, comm):
+    c1 = hv.Curve([])
+
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore", PanelDeprecationWarning)
+        pane = pn.panel(c1, backend='bokeh', background='red',
+                        css_classes=['test_class'])
+    model = pane.get_root(document, comm=comm)
+
+    assert model.styles["background"] == 'red'
+    assert model.children[0].css_classes == ['test_class']
+
 @hv_available
 def test_holoviews_property_override(document, comm):
     c1 = hv.Curve([])
 
-    pane = pn.panel(c1, backend='bokeh', background='red',
+    pane = pn.panel(c1, backend='bokeh',
+                styles={'background': 'red'},
                 css_classes=['test_class'])
     model = pane.get_root(document, comm=comm)
 
-    assert model.background == 'red'
-    assert model.css_classes == ['test_class']
+    assert model.styles["background"] == 'red'
+    assert model.children[0].css_classes == ['test_class']
