@@ -4,7 +4,7 @@ import param
 import pytest
 
 from bokeh.models import (
-    AutocompleteInput as BkAutocompleteInput, Button, CheckboxGroup,
+    AutocompleteInput as BkAutocompleteInput, Button, Checkbox,
     Column as BkColumn, Div, MultiSelect, RangeSlider as BkRangeSlider,
     Row as BkRow, Select, Slider, Tabs as BkTabs, TextInput,
     TextInput as BkTextInput, Toggle,
@@ -13,6 +13,7 @@ from bokeh.models import (
 from panel import config
 from panel.io.state import set_curdoc, state
 from panel.layout import Row, Tabs
+from panel.models import HTML as BkHTML
 from panel.pane import (
     HTML, Bokeh, Matplotlib, PaneBase, Str, panel,
 )
@@ -117,9 +118,9 @@ def test_get_root(document, comm):
     assert isinstance(model, BkColumn)
     assert len(model.children) == 1
 
-    div = model.children[0]
-    assert isinstance(div, Div)
-    assert div.text == '<b>'+test.name[:-5]+'</b>'
+    html = model.children[0]
+    assert isinstance(html, Div)
+    assert html.text == '<b>'+test.name[:-5]+'</b>'
 
 
 def test_single_param(document, comm):
@@ -205,14 +206,14 @@ def test_boolean_param(document, comm):
     model = test_pane.get_root(document, comm=comm)
 
     checkbox = model.children[1]
-    assert isinstance(checkbox, CheckboxGroup)
-    assert checkbox.labels == ['A']
-    assert checkbox.active == []
+    assert isinstance(checkbox, Checkbox)
+    assert checkbox.label == 'A'
+    assert checkbox.active == False
     assert checkbox.disabled == False
 
     # Check changing param value updates widget
     test.a = True
-    assert checkbox.active == [0]
+    assert checkbox.active == True
 
     # Check changing param attribute updates widget
     a_param = test.param['a']
@@ -223,7 +224,7 @@ def test_boolean_param(document, comm):
     test_pane._cleanup(model)
     a_param.constant = False
     test.a = False
-    assert checkbox.active == [0]
+    assert checkbox.active == True
     assert checkbox.disabled == True
 
 
@@ -378,7 +379,7 @@ def test_list_selector_param(document, comm):
 
 def test_action_param(document, comm):
     class Test(param.Parameterized):
-        a = param.Action(lambda x: setattr(x, 'b', 2))
+        a = param.Action(default=lambda x: setattr(x, 'b', 2))
         b = param.Number(default=1)
 
     test = Test()
@@ -445,7 +446,7 @@ def test_explicit_params(document, comm):
     model = test_pane.get_root(document, comm=comm)
 
     assert len(model.children) == 2
-    assert isinstance(model.children[1], CheckboxGroup)
+    assert isinstance(model.children[1], Checkbox)
 
 
 def test_param_precedence(document, comm):
@@ -961,7 +962,7 @@ def test_expand_param_subobject_tabs(document, comm):
 
 def test_param_js_callbacks(document, comm):
     class JsButton(param.Parameterized):
-        param_btn = param.Action(lambda self: print('Action Python Response'), label='Action')
+        param_btn = param.Action(default=lambda self: print('Action Python Response'), label='Action')
 
     param_button = Param(JsButton())
     code = "console.log('Action button clicked')"
@@ -1082,7 +1083,7 @@ def test_param_function_pane_defer_load(document, comm):
     assert len(row.children) == 1
     model = row.children[0]
     assert pane._models[row.ref['id']][0] is row
-    assert isinstance(model, Div)
+    assert isinstance(model, BkHTML)
     assert model.text == '&lt;pre&gt; &lt;/pre&gt;'
 
     # Test on_load
@@ -1230,7 +1231,7 @@ def test_param_method_pane_subobject(document, comm):
     assert model.text == '42'
 
     # Ensure that subobject is being watched
-    watchers = pane._callbacks
+    watchers = pane._internal_callbacks
     assert any(w.inst is subobject for w in watchers)
     assert pane._models[row.ref['id']][0] is row
 
@@ -1238,7 +1239,7 @@ def test_param_method_pane_subobject(document, comm):
     new_subobject = View(name='Nested', a=42)
     test.b = new_subobject
     assert pane._models[row.ref['id']][0] is row
-    watchers = pane._callbacks
+    watchers = pane._internal_callbacks
     assert not any(w.inst is subobject for w in watchers)
     assert any(w.inst is new_subobject for w in watchers)
 
