@@ -166,7 +166,7 @@ def test_autoload_js(port):
     r = requests.get(f"http://localhost:{port}/{app_name}/autoload.js?{args}")
 
     assert r.status_code == 200
-    assert f"http://localhost:{port}/static/extensions/panel/css/alerts.css" in r.content.decode('utf-8')
+    assert f"http://localhost:{port}/static/extensions/panel/panel.min.js" in r.content.decode('utf-8')
 
 
 def test_server_async_callbacks(port):
@@ -292,6 +292,9 @@ def test_serve_config_per_session_state():
     assert CSS2 in r2
 
 
+# This test seem to fail if run after:
+# - test_server_async_local_state_nested_tasks
+# - test_server_async_local_state
 def test_server_session_info(port):
     with config.set(session_history=-1):
         html = Markdown('# Title')
@@ -453,6 +456,61 @@ def test_server_schedule_at_callable(port):
 
     server.stop()
 
+@pytest.mark.xdist_group(name="server")
+def test_server_reuse_sessions(port, reuse_sessions):
+    def app(counts=[0]):
+        content = f'# Count {counts[0]}'
+        counts[0] += 1
+        return content
+
+    serve(app, port=port, threaded=True, show=False)
+
+    # Wait for server to start
+    time.sleep(1)
+
+    r1 = requests.get(f"http://localhost:{port}/")
+    r2 = requests.get(f"http://localhost:{port}/")
+
+    assert len(state._sessions) == 1
+    assert ('/', 'default') in state._sessions
+
+    session = state._sessions[('/', 'default')]
+
+    assert session.token in r1.content.decode('utf-8')
+    assert session.token not in r2.content.decode('utf-8')
+
+
+@pytest.mark.xdist_group(name="server")
+def test_server_reuse_sessions_with_session_key_func(port, reuse_sessions):
+    config.session_key_func = lambda r: (r.path, r.arguments.get('arg', [''])[0])
+    def app(counts=[0]):
+        if 'arg' in state.session_args:
+            title = state.session_args['arg'][0].decode('utf-8')
+        else:
+            title = 'Empty'
+        content = f"# Count {counts[0]}"
+        tmpl = BootstrapTemplate(title=title)
+        tmpl.main.append(content)
+        counts[0] += 1
+        return tmpl
+
+    serve(app, port=port, threaded=True, show=False)
+
+    # Wait for server to start
+    time.sleep(1)
+
+    r1 = requests.get(f"http://localhost:{port}/?arg=foo")
+    r2 = requests.get(f"http://localhost:{port}/?arg=bar")
+
+    assert len(state._sessions) == 2
+    assert ('/', b'foo') in state._sessions
+    assert ('/', b'bar') in state._sessions
+
+    session1, session2 = state._sessions.values()
+    assert session1.token in r1.content.decode('utf-8')
+    assert session2.token in r2.content.decode('utf-8')
+
+
 @pytest.mark.xdist_group(name="server")
 def test_show_server_info(html_server_session, markdown_server_session):
     *_, html_port = html_server_session
@@ -754,6 +812,8 @@ def test_server_template_custom_resources_with_subpath_and_prefix_relative_url(p
 
 class CustomComponent(ReactiveHTML):
 
+    _extension_name = 'custom'
+
     __css__ = ['./assets/custom.css']
 
 
