@@ -1,10 +1,15 @@
 from collections import OrderedDict
 
+import param
+
 from bokeh.models import Div
 
+from panel.depends import bind
 from panel.io.notebook import render_mimebundle
 from panel.pane import PaneBase
-from panel.util import get_method_owner, abbreviated_repr, parse_query
+from panel.util import (
+    abbreviated_repr, extract_dependencies, get_method_owner, parse_query,
+)
 
 
 def test_get_method_owner_class():
@@ -16,6 +21,50 @@ def test_get_method_owner_instance():
     assert get_method_owner(div.update) is div
 
 
+def test_get_function_dependencies():
+    class Test(param.Parameterized):
+        a = param.Parameter()
+
+    assert extract_dependencies(bind(lambda a: a, Test.param.a)) == [Test.param.a]
+
+
+def test_get_parameterized_dependencies():
+    class Test(param.Parameterized):
+
+        a = param.Parameter()
+        b = param.Parameter()
+
+        @param.depends('a')
+        def dep_a(self):
+            return
+
+        @param.depends('dep_a', 'b')
+        def dep_ab(self):
+            return
+
+    test = Test()
+
+    assert extract_dependencies(test.dep_a) == [test.param.a]
+    assert extract_dependencies(test.dep_ab) == [test.param.a, test.param.b]
+
+
+def test_get_parameterized_subobject_dependencies():
+    class A(param.Parameterized):
+
+        value = param.Parameter()
+
+    class B(param.Parameterized):
+
+        a = param.ClassSelector(default=A(), class_=A)
+
+        @param.depends('a.value')
+        def dep_a_value(self):
+            return
+
+    test = B()
+
+    assert extract_dependencies(test.dep_a_value) == [test.a.param.value]
+
 def test_render_mimebundle(document, comm):
     div = Div()
     data, metadata = render_mimebundle(div, document, comm)
