@@ -4,6 +4,7 @@ import pytest
 from panel import config
 from panel.interact import interactive
 from panel.pane import Markdown, Str, panel
+from panel.param import ParamMethod
 from panel.viewable import Viewable, Viewer
 
 from .util import jb_available
@@ -12,6 +13,22 @@ all_viewables = [w for w in param.concrete_descendents(Viewable).values()
                if not w.__name__.startswith('_') and
                not issubclass(w, interactive)]
 
+
+class ExampleViewer(Viewer):
+    value = param.String()
+
+    def __panel__(self):
+        return self.value
+
+
+class ExampleViewerWithDeps(Viewer):
+    value = param.String()
+
+    @param.depends('value')
+    def __panel__(self):
+        return self.value
+
+
 @jb_available
 def test_viewable_ipywidget():
     pane = Str('A')
@@ -19,7 +36,6 @@ def test_viewable_ipywidget():
         data, metadata = pane._repr_mimebundle_()
     assert 'application/vnd.jupyter.widget-view+json' in data
 
-
 @pytest.mark.parametrize('viewable', all_viewables)
 def test_viewable_signature(viewable):
     from inspect import Parameter, signature
@@ -30,7 +46,6 @@ def test_viewable_signature(viewable):
     except Exception:
         assert parameters['params'] == Parameter('params', Parameter.VAR_KEYWORD)
 
-
 def test_Viewer_not_initialized():
     class Test(Viewer):
         def __panel__(self):
@@ -44,16 +59,35 @@ def test_Viewer_not_initialized():
     assert test.object == "# Test"
 
 def test_viewer_wraps_panel():
-    class TestViewer(Viewer):
-        value = param.String()
+    tv = ExampleViewer(value="hello")
 
-        def __panel__(self):
-            return self.value
+    view = tv._create_view()
+    assert isinstance(view, Markdown)
+    assert view.object == "hello"
 
-    tv = TestViewer(value="hello")
+def test_viewer_wraps_panel_with_deps(document, comm):
+    tv = ExampleViewerWithDeps(value="hello")
 
-    assert isinstance(tv._create_view(), Markdown)
+    view = tv._create_view()
 
+    view.get_root(document, comm)
+
+    assert isinstance(view, ParamMethod)
+    assert view._pane.object == "hello"
+
+    tv.value = "goodbye"
+
+    assert view._pane.object == "goodbye"
+
+def test_viewer_with_deps_resolved_by_panel_func(document, comm):
+    tv = ExampleViewerWithDeps(value="hello")
+
+    view = panel(tv)
+
+    view.get_root(document, comm)
+
+    assert isinstance(view, ParamMethod)
+    assert view._pane.object == "hello"
 
 def test_non_viewer_class():
     # This test checks that a class with __panel__ (other than Viewer)
