@@ -62,6 +62,7 @@ pn.Row(button, tabulator).servable();
 
 csv_app = """
 import pandas as pd
+import sys
 import panel as pn
 
 df = pd.read_csv('https://raw.githubusercontent.com/holoviz/panel/main/examples/assets/occupancy.csv')
@@ -153,7 +154,7 @@ def wait_for_app(launch_app, app, page, runtime, wait=True, **kwargs):
 
     convert_apps(
         [app_path], app_path.parent, runtime=runtime, build_pwa=False,
-        prerender=False, panel_version='local', **kwargs
+        prerender=False, panel_version='local', inline=True, **kwargs
     )
 
     msgs = []
@@ -161,7 +162,7 @@ def wait_for_app(launch_app, app, page, runtime, wait=True, **kwargs):
 
     page.goto(f"http://localhost:8123/{app_path.name[:-3]}.html")
 
-    cls = f'bk pn-loading {config.loading_spinner}'
+    cls = f'pn-loading {config.loading_spinner}'
     expect(page.locator('body')).to_have_class(cls)
     if wait:
         expect(page.locator('body')).not_to_have_class(cls, timeout=90 * 1000)
@@ -178,11 +179,11 @@ def test_pyodide_test_error_handling_worker(page, launch_app):
 def test_pyodide_test_convert_button_app(page, runtime, launch_app):
     msgs = wait_for_app(launch_app, button_app, page, runtime)
 
-    expect(page.locator(".bk.bk-clearfix")).to_have_text('0')
+    expect(page.locator('pre:not([class])')).to_have_text('0')
 
-    page.click('.bk.bk-btn')
+    page.click('.bk-btn')
 
-    expect(page.locator(".bk.bk-clearfix")).to_have_text('1')
+    expect(page.locator('pre:not([class])')).to_have_text('1')
 
     assert [msg for msg in msgs if msg.type == 'error' and 'favicon' not in msg.location['url']] == []
 
@@ -190,12 +191,12 @@ def test_pyodide_test_convert_button_app(page, runtime, launch_app):
 def test_pyodide_test_convert_slider_app(page, runtime, launch_app):
     msgs = wait_for_app(launch_app, slider_app, page, runtime)
 
-    expect(page.locator(".bk.bk-clearfix")).to_have_text('0.0')
+    expect(page.locator('pre:not([class])')).to_have_text('0.0')
 
     page.click('.noUi-handle')
     page.keyboard.press('ArrowRight')
 
-    expect(page.locator(".bk.bk-clearfix")).to_have_text('0.1')
+    expect(page.locator('pre:not([class])')).to_have_text('0.1')
 
     assert [msg for msg in msgs if msg.type == 'error' and 'favicon' not in msg.location['url']] == []
 
@@ -210,7 +211,7 @@ def test_pyodide_test_convert_custom_config(page, runtime, launch_app):
 def test_pyodide_test_convert_tabulator_app(page, runtime, launch_app):
     msgs = wait_for_app(launch_app, tabulator_app, page, runtime)
 
-    page.click('.bk.bk-btn')
+    page.click('.bk-btn')
 
     time.sleep(1)
 
