@@ -39,7 +39,7 @@ def test_material_template_updates(page, port):
 
     page.goto(f"http://localhost:{port}")
 
-    assert page.text_content(".bk.markdown") == 'Initial'
+    assert page.locator(".markdown").locator("div").text_content() == 'Initial\n'
     md.object = 'Updated'
     time.sleep(0.1)
-    assert page.text_content(".bk.markdown") == 'Updated'
+    assert page.locator(".markdown").locator("div").text_content() == 'Updated\n'
