@@ -1,6 +1,9 @@
 import pytest
 
-from panel.widgets.indicators import Dial, Gauge, Number, Tqdm
+from panel.widgets.indicators import (
+    Dial, Gauge, Number, Tqdm,
+)
+
 
 def test_number_none(document, comm):
     number = Number(value=None, name='Value')
@@ -101,7 +104,7 @@ def test_gauge_bounds():
 
 def test_tqdm_color():
     tqdm = Tqdm()
-    tqdm.text_pane.style={'color': 'green'}
+    tqdm.text_pane.styles={'color': 'green'}
     for _ in tqdm(range(0,2)):
         pass
-    assert tqdm.text_pane.style["color"]=="green"
+    assert tqdm.text_pane.styles["color"]=="green"
