@@ -1,6 +1,8 @@
 """Tests of the Tqdm indicator"""
 import time
 
+import numpy as np
+import pandas as pd
 import pytest
 
 from tqdm.contrib.concurrent import process_map
@@ -56,15 +58,10 @@ def test_tqdm_color():
     for index in tqdm(range(0, 3), colour='red'):
         pass
 
-    assert tqdm.text_pane.style == {'color': 'red'}
+    assert tqdm.text_pane.styles == {'color': 'red'}
 
 
 def get_tqdm_app():
-    import time
-
-    import numpy as np
-    import pandas as pd
-
     tqdm = Tqdm(layout="row", sizing_mode="stretch_width")
 
     def run(*events):
