@@ -21,7 +21,6 @@ import uuid
 from functools import partial
 from typing import (
     IO, TYPE_CHECKING, Any, Callable, ClassVar, Dict, List, Mapping, Optional,
-    TypeVar,
 )
 
 import param  # type: ignore
@@ -29,16 +28,18 @@ import param  # type: ignore
 from bokeh.document import Document
 from bokeh.resources import Resources
 from jinja2 import Template
-from pyviz_comms import Comm, JupyterCommManager  # type: ignore
+from pyviz_comms import Comm  # type: ignore
 
+from ._param import Align, Aspect, Margin
 from .config import config, panel_extension
 from .io import serve
-from .io.document import init_doc
+from .io.document import create_doc_if_none_exists, init_doc
 from .io.embed import embed_state
 from .io.loading import start_loading_spinner, stop_loading_spinner
 from .io.model import add_to_doc, patch_cds_msg
 from .io.notebook import (
-    ipywidget, render_mimebundle, render_model, show_embed, show_server,
+    JupyterCommManagerBinary as JupyterCommManager, ipywidget, render_embed,
+    render_mimebundle, render_model, show_server,
 )
 from .io.save import save
 from .io.state import curdoc_locked, state
@@ -59,12 +60,12 @@ class Layoutable(param.Parameterized):
     for all Panel components with a visual representation.
     """
 
-    align = param.ClassSelector(default='start', class_=(str, tuple), doc="""
+    align = Align(default='start', doc="""
         Whether the object should be aligned with the start, end or
         center of its container. If set as a tuple it will declare
         (vertical, horizontal) alignment.""")
 
-    aspect_ratio = param.Parameter(default=None, doc="""
+    aspect_ratio = Aspect(default=None, doc="""
         Describes the proportional relationship between component's
         width and height.  This works if any of component's dimensions
         are flexible in size. If set to a number, ``width / height =
@@ -76,12 +77,11 @@ class Layoutable(param.Parameterized):
     background = param.Parameter(default=None, doc="""
         Background color of the component.""")
 
-    css_classes = param.List(default=None, doc="""
+    css_classes = param.List(default=[], doc="""
         CSS classes to apply to the layout.""")
 
-    width = param.Integer(default=None, bounds=(0, None), doc="""
-        The width of the component (in pixels). This can be either
-        fixed or preferred width, depending on width sizing policy.""")
+    design = param.ObjectSelector(default=None, objects=[], doc="""
+        The design system to use to style components.""")
 
     height = param.Integer(default=None, bounds=(0, None), doc="""
         The height of the component (in pixels).  This can be either
@@ -99,11 +99,28 @@ class Layoutable(param.Parameterized):
     max_height = param.Integer(default=None, bounds=(0, None), doc="""
         Minimal height of the component (in pixels) if height is adjustable.""")
 
-    margin = param.Parameter(default=5, doc="""
+    margin = Margin(default=0, doc="""
         Allows to create additional space around the component. May
         be specified as a two-tuple of the form (vertical, horizontal)
         or a four-tuple (top, right, bottom, left).""")
 
+    styles = param.Dict(default={}, doc="""
+        Dictionary of CSS rules to apply to DOM node wrapping the
+        component.""")
+
+    stylesheets = param.List(default=[], doc="""
+        List of stylesheets defined as URLs pointing to .css files
+        or raw CSS defined as a string.""")
+
+    tags = param.List(default=[], doc="""
+        List of arbitrary tags to add to the component.
+        Can be useful for templating or for storing metadata on
+        the model.""")
+
+    width = param.Integer(default=None, bounds=(0, None), doc="""
+        The width of the component (in pixels). This can be either
+        fixed or preferred width, depending on width sizing policy.""")
+
     width_policy = param.ObjectSelector(
         default="auto", objects=['auto', 'fixed', 'fit', 'min', 'max'], doc="""
         Describes how the component should maintain its width.
@@ -226,6 +243,35 @@ class Layoutable(param.Parameterized):
     __abstract = True
 
     def __init__(self, **params):
+        sizing_mode = params.get('sizing_mode')
+        if (sizing_mode in ('stretch_width', 'scale_width', 'stretch_both', 'scale_both') and
+            params.get('width') is not None):
+            error = (
+                f"Providing a width-responsive sizing_mode ({params['sizing_mode']!r}) "
+                "and a fixed width is not supported. Converting fixed width to min_width. "
+                "If you intended the component to be fully width-responsive remove the height"
+                "setting, otherwise change it to min_height."
+            )
+            if config.layout_compatibility == 'warn':
+                error += ' To error on the incorrect specification disable the config.layout_compatibility option.'
+                self.param.warning(error)
+            else:
+                raise ValueError(error)
+            params['min_width'] = params.pop('width')
+        if (sizing_mode in ('stretch_height', 'scale_height', 'stretch_both', 'scale_both') and
+            params.get('height') is not None):
+            error = (
+                f"Providing a height-responsive sizing_mode ({params['sizing_mode']!r}) "
+                "and a fixed height is not supported. Converting fixed height to min_height. "
+                "If you intended the component to be fully height-responsive remove the height "
+                "setting, otherwise change it to min_height."
+            )
+            if config.layout_compatibility == 'warn':
+                error += ' To error on the incorrect specification disable the config.layout_compatibility option.'
+                self.param.warning(error)
+            else:
+                raise ValueError(error)
+            params['min_height'] = params.pop('height')
         if (params.get('width') is not None and
             params.get('height') is not None and
             params.get('width_policy') is None and
@@ -245,12 +291,13 @@ class Layoutable(param.Parameterized):
                 params['sizing_mode'] = config.sizing_mode
             elif config.sizing_mode == 'stretch_height' and 'height' not in params:
                 params['sizing_mode'] = config.sizing_mode
+        if 'design' not in params and self.param.design.default is None:
+            params['design'] = config.design
         super().__init__(**params)
 
 
-_Self = TypeVar('_Self', bound='ServableMixin')
 
-class ServableMixin(object):
+class ServableMixin:
     """
     Mixin to define methods shared by objects which can served.
     """
@@ -282,47 +329,10 @@ class ServableMixin(object):
         else:
             loc_model = loc._get_model(doc, root)
         loc_model.name = 'location'
+        doc.on_session_destroyed(loc._server_destroy) # type: ignore
         doc.add_root(loc_model)
         return loc
 
-    def _on_msg(self, ref: str, manager, msg) -> None:
-        """
-        Handles Protocol messages arriving from the client comm.
-        """
-        root, doc, comm = state._views[ref][1:]
-        patch_cds_msg(root, msg)
-        held = doc.callbacks.hold_value
-        patch = manager.assemble(msg)
-        doc.hold()
-        patch.apply_to_document(doc, comm.id if comm else None)
-        doc.unhold()
-        if held:
-            doc.hold(held)
-
-    def _on_error(self, ref: str, error: Exception) -> None:
-        if ref not in state._handles or config.console_output in [None, 'disable']:
-            return
-        handle, accumulator = state._handles[ref]
-        formatted = '\n<pre>'+escape(traceback.format_exc())+'</pre>\n'
-        if config.console_output == 'accumulate':
-            accumulator.append(formatted)
-        elif config.console_output == 'replace':
-            accumulator[:] = [formatted]
-        if accumulator:
-            handle.update({'text/html': '\n'.join(accumulator)}, raw=True)
-
-    def _on_stdout(self, ref: str, stdout: Any) -> None:
-        if ref not in state._handles or config.console_output is [None, 'disable']:
-            return
-        handle, accumulator = state._handles[ref]
-        formatted = ["%s</br>" % o for o in stdout]
-        if config.console_output == 'accumulate':
-            accumulator.extend(formatted)
-        elif config.console_output == 'replace':
-            accumulator[:] = formatted
-        if accumulator:
-            handle.update({'text/html': '\n'.join(accumulator)}, raw=True)
-
     #----------------------------------------------------------------
     # Public API
     #----------------------------------------------------------------
@@ -330,7 +340,7 @@ class ServableMixin(object):
     def servable(
         self, title: Optional[str] = None, location: bool | 'Location' = True,
         area: str = 'main', target: Optional[str] = None
-    ) -> _Self:
+    ) -> 'ServableMixin':
         """
         Serves the object or adds it to the configured
         pn.state.template if in a `panel serve` context, writes to the
@@ -378,9 +388,14 @@ class ServableMixin(object):
                     template.header.append(self)
             else:
                 self.server_doc(title=title, location=location) # type: ignore
-        elif state._is_pyodide:
-            from .io.pyodide import _get_pyscript_target, write
-            target = target or _get_pyscript_target()
+        elif state._is_pyodide and 'pyodide_kernel' not in sys.modules:
+            from .io.pyodide import _IN_WORKER, _get_pyscript_target, write
+            if _IN_WORKER:
+                return self
+            try:
+                target = target or _get_pyscript_target()
+            except Exception:
+                target = None
             if target is not None:
                 asyncio.create_task(write(target, self))
         return self
@@ -430,7 +445,66 @@ class ServableMixin(object):
         )
 
 
-class Renderable(param.Parameterized):
+class MimeRenderMixin:
+    """
+    Mixin class to allow rendering and syncing objects in notebook
+    contexts.
+    """
+
+    def _on_msg(self, ref: str, manager, msg) -> None:
+        """
+        Handles Protocol messages arriving from the client comm.
+        """
+        root, doc, comm = state._views[ref][1:]
+        patch_cds_msg(root, msg)
+        held = doc.callbacks.hold_value
+        patch = manager.assemble(msg)
+        doc.hold()
+        patch.apply_to_document(doc, comm.id if comm else None)
+        doc.unhold()
+        if held:
+            doc.hold(held)
+
+    def _on_error(self, ref: str, error: Exception) -> None:
+        if ref not in state._handles or config.console_output in [None, 'disable']:
+            return
+        handle, accumulator = state._handles[ref]
+        formatted = '\n<pre>'+escape(traceback.format_exc())+'</pre>\n'
+        if config.console_output == 'accumulate':
+            accumulator.append(formatted)
+        elif config.console_output == 'replace':
+            accumulator[:] = [formatted]
+        if accumulator:
+            handle.update({'text/html': '\n'.join(accumulator)}, raw=True)
+
+    def _on_stdout(self, ref: str, stdout: Any) -> None:
+        if ref not in state._handles or config.console_output is [None, 'disable']:
+            return
+        handle, accumulator = state._handles[ref]
+        formatted = ["%s</br>" % o for o in stdout]
+        if config.console_output == 'accumulate':
+            accumulator.extend(formatted)
+        elif config.console_output == 'replace':
+            accumulator[:] = formatted
+        if accumulator:
+            handle.update({'text/html': '\n'.join(accumulator)}, raw=True)
+
+    def _render_mimebundle(self, model: Model, doc: Document, comm: Comm, location: Location | None = None):
+        from .models.comm_manager import CommManager
+
+        ref = model.ref['id']
+        manager = CommManager(comm_id=comm.id, plot_id=ref)
+        client_comm = state._comm_manager.get_client_comm(
+            on_msg=partial(self._on_msg, ref, manager),
+            on_error=partial(self._on_error, ref),
+            on_stdout=partial(self._on_stdout, ref)
+        )
+        self._comms[ref] = (comm, client_comm)
+        manager.client_comm_id = client_comm.id
+        return render_mimebundle(model, doc, comm, manager, location)
+
+
+class Renderable(param.Parameterized, MimeRenderMixin):
     """
     Baseclass for objects which can be rendered to a Bokeh model.
 
@@ -441,12 +515,12 @@ class Renderable(param.Parameterized):
     __abstract = True
 
     def __init__(self, **params):
-        super().__init__(**params)
-        self._callbacks = []
+        self._internal_callbacks = []
         self._documents = {}
         self._models = {}
         self._comms = {}
         self._kernels = {}
+        super().__init__(**params)
         self._found_links = set()
         self._logger = logging.getLogger(f'{__name__}.{type(self).__name__}')
 
@@ -536,6 +610,10 @@ class Renderable(param.Parameterized):
         if ref in state._views:
             del state._views[ref]
 
+    def __repr__(self, depth: int = 0) -> str:
+        return '{cls}({params})'.format(cls=type(self).__name__,
+                                        params=', '.join(param_reprs(self)))
+
     def get_root(
         self, doc: Optional[Document] = None, comm: Optional[Comm] = None,
         preprocess: bool = True
@@ -556,12 +634,23 @@ class Renderable(param.Parameterized):
         -------
         Returns the bokeh model corresponding to this panel object
         """
-        doc = init_doc(doc)
-        root = self._get_model(doc, comm=comm)
-        if preprocess:
-            self._preprocess(root)
+        doc = create_doc_if_none_exists(doc)
+        if self._design and (comm or (state._is_pyodide and not doc.session_context)):
+            wrapper = self._design._wrapper(self)
+            if wrapper is self:
+                root = self._get_model(doc, comm=comm)
+                if preprocess:
+                    self._preprocess(root)
+            else:
+                root = wrapper.get_root(doc, comm, preprocess)
+            root_view = wrapper
+        else:
+            root = self._get_model(doc, comm=comm)
+            root_view = self
+            if preprocess:
+                self._preprocess(root)
         ref = root.ref['id']
-        state._views[ref] = (self, root, doc, comm)
+        state._views[ref] = (root_view, root, doc, comm)
         return root
 
 
@@ -585,9 +674,25 @@ class Viewable(Renderable, Layoutable, ServableMixin):
         hooks = params.pop('hooks', [])
         super().__init__(**params)
         self._hooks = hooks
+
         self._update_loading()
-        watcher = self.param.watch(self._update_loading, 'loading')
-        self._callbacks.append(watcher)
+        self._update_background()
+        self._update_design()
+        self._internal_callbacks.extend([
+            self.param.watch(self._update_background, 'background'),
+            self.param.watch(self._update_design, 'design'),
+            self.param.watch(self._update_loading, 'loading')
+        ])
+
+    def _update_design(self, *_):
+        from .theme import Design
+        from .theme.native import Native
+        if isinstance(self.design, Design):
+            self._design = self.design
+        elif self.design:
+            self._design = self.design(theme=config.theme)
+        else:
+            self._design = Native(theme=config.theme)
 
     def _update_loading(self, *_) -> None:
         if self.loading:
@@ -595,12 +700,37 @@ class Viewable(Renderable, Layoutable, ServableMixin):
         else:
             stop_loading_spinner(self)
 
-    def __repr__(self, depth: int = 0) -> str:
-        return '{cls}({params})'.format(cls=type(self).__name__,
-                                        params=', '.join(param_reprs(self)))
+    def _update_background(self, *_) -> None:
+        if self.background == self.styles.get("background", None) or self.background is None:
+            return
 
-    def __str__(self) -> str:
-        return self.__repr__()
+        # Warning
+        prev = f'{type(self).name}(..., background={self.background!r})'
+        new = f"{type(self).name}(..., styles={{'background': {self.background!r}}})"
+        deprecated("1.1", prev, new)
+
+        self.styles = dict(self.styles, background=self.background)
+
+    def _render_model(self, doc: Optional[Document] = None, comm: Optional[Comm] = None) -> 'Model':
+        if doc is None:
+            doc = Document()
+        if comm is None:
+            comm = state._comm_manager.get_server_comm()
+        model = self.get_root(doc, comm)
+
+        if self._design and self._design.theme.bokeh_theme:
+            doc.theme = self._design.theme.bokeh_theme
+
+        if config.embed:
+            embed_state(self, model, doc,
+                        json=config.embed_json,
+                        json_prefix=config.embed_json_prefix,
+                        save_path=config.embed_save_path,
+                        load_path=config.embed_load_path,
+                        progress=False)
+        else:
+            add_to_doc(model, doc)
+        return model
 
     def _repr_mimebundle_(self, include=None, exclude=None):
         if state._is_pyodide:
@@ -666,30 +796,25 @@ class Viewable(Renderable, Layoutable, ServableMixin):
         else:
             location = None
 
-        from IPython.display import display
-
-        from .models.comm_manager import CommManager
-
         doc = Document()
         comm = state._comm_manager.get_server_comm()
         model = self._render_model(doc, comm)
-        ref = model.ref['id']
-        manager = CommManager(comm_id=comm.id, plot_id=ref)
-        client_comm = state._comm_manager.get_client_comm(
-            on_msg=partial(self._on_msg, ref, manager),
-            on_error=partial(self._on_error, ref),
-            on_stdout=partial(self._on_stdout, ref)
-        )
-        self._comms[ref] = (comm, client_comm)
-        manager.client_comm_id = client_comm.id
+        if config.embed:
+            return render_model(model)
 
-        if config.console_output != 'disable':
+        bundle, meta = self._render_mimebundle(model, doc, comm, location)
+
+        if config.console_output != 'disable' and not state._is_pyodide:
+            from IPython.display import display
+
+            ref = model.ref['id']
             handle = display(display_id=uuid.uuid4().hex)
             state._handles[ref] = (handle, [])
 
-        if config.embed:
-            return render_model(model)
-        return render_mimebundle(model, doc, comm, manager, location)
+        return bundle, meta
+
+    def __str__(self) -> str:
+        return self.__repr__()
 
     #----------------------------------------------------------------
     # Public API
@@ -707,15 +832,18 @@ class Viewable(Renderable, Layoutable, ServableMixin):
         -------
         Cloned Viewable object
         """
-        inherited = {p: v for p, v in self.param.values().items()
-                     if not self.param[p].readonly}
+        inherited = {
+            p: v for p, v in self.param.values().items()
+            if not self.param[p].readonly and v is not self.param[p].default
+            and not (v is None and not self.param[p].allow_None)
+        }
         return type(self)(**dict(inherited, **params))
 
     def pprint(self) -> None:
         """
         Prints a compositional repr of the class.
         """
-        deprecated('1.0', f'{type(self).__name__}.pprint', 'print')
+        deprecated('1.1', f'{type(self).__name__}.pprint', 'print')
         print(self)
 
     def select(
@@ -753,7 +881,7 @@ class Viewable(Renderable, Layoutable, ServableMixin):
         port: int (optional, default=0)
           Allows specifying a specific port
         """
-        deprecated('1.0', f'{type(self).__name__}.app', 'panel.io.notebook.show_server')
+        deprecated('1.1', f'{type(self).__name__}.app', 'panel.io.notebook.show_server')
         return show_server(self, notebook_url, port)
 
     def embed(
@@ -786,7 +914,7 @@ class Viewable(Renderable, Layoutable, ServableMixin):
         states: dict (default={})
           A dictionary specifying the widget values to embed for each widget
         """
-        show_embed(
+        return render_embed(
             self, max_states, max_opts, json, json_prefix, save_path,
             load_path, progress, states
         )
@@ -871,7 +999,18 @@ class Viewable(Renderable, Layoutable, ServableMixin):
         if title or doc.title == 'Bokeh Application':
             title = title or 'Panel Application'
             doc.title = title
-        model = self.get_root(doc)
+
+        if self._design:
+            wrapper = self._design._wrapper(self)
+            if wrapper is self:
+                model = self.get_root(doc)
+            else:
+                model = wrapper.get_root(doc)
+                ref = model.ref['id']
+                state._views[ref] = (wrapper, model, doc, None)
+        else:
+            model = self.get_root(doc)
+
         doc.on_session_destroyed(state._destroy_session)
         doc.on_session_destroyed(self._server_destroy) # type: ignore
         self._documents[doc] = model
@@ -882,6 +1021,10 @@ class Viewable(Renderable, Layoutable, ServableMixin):
             notification_model = state.notifications._get_model(doc, model)
             notification_model.name = 'notifications'
             doc.add_root(notification_model)
+        if config.browser_info and doc is state.curdoc:
+            browser_model = state.browser_info._get_model(doc, model)
+            browser_model.name = 'browser_info'
+            doc.add_root(browser_model)
         return doc
 
 
