@@ -23,7 +23,7 @@ import sys
 
 from math import pi
 from typing import (
-    TYPE_CHECKING, ClassVar, Dict, List, Mapping, Optional, Type,
+    TYPE_CHECKING, ClassVar, Dict, List, Mapping, Optional, Tuple, Type,
 )
 
 import numpy as np
@@ -33,6 +33,7 @@ from bokeh.models import ColumnDataSource, FixedTicker
 from bokeh.plotting import figure
 from tqdm.asyncio import tqdm as _tqdm
 
+from ..io.resources import CDN_DIST
 from ..layout import Column, Panel, Row
 from ..models import (
     HTML, Progress as _BkProgress, TrendIndicator as _BkTrendIndicator,
@@ -63,11 +64,11 @@ class Indicator(Widget):
         'fixed', 'stretch_width', 'stretch_height', 'stretch_both',
         'scale_width', 'scale_height', 'scale_both', None])
 
-    __abstract = True
+    _linked_properties: ClassVar[Tuple[str]] = ()
 
-    def _filter_properties(self, properties):
-        "Indicators are solely display units so we do not need to sync properties."
-        return []
+    _rename: ClassVar[Mapping[str, str | None]] = {'name': None}
+
+    __abstract = True
 
 
 class BooleanIndicator(Indicator):
@@ -111,10 +112,10 @@ class BooleanStatus(BooleanIndicator):
     value = param.Boolean(default=False, doc="""
         Whether the indicator is active or not.""")
 
-    _rename: ClassVar[Mapping[str, str | None]] = {}
-
     _source_transforms: ClassVar[Mapping[str, str | None]] = {'value': None, 'color': None}
 
+    _stylesheets: ClassVar[List[str]] = [f'{CDN_DIST}css/booleanstatus.css']
+
     _widget_type: ClassVar[Type[Model]] = HTML
 
     def _process_param_change(self, msg):
@@ -157,10 +158,10 @@ class LoadingSpinner(BooleanIndicator):
     value = param.Boolean(default=False, doc="""
         Whether the indicator is active or not.""")
 
-    _rename: ClassVar[Mapping[str, str | None]] = {}
-
     _source_transforms: ClassVar[Mapping[str, str | None]] = {'value': None, 'color': None, 'bgcolor': None}
 
+    _stylesheets: ClassVar[List[str]] = [f'{CDN_DIST}css/loadingspinner.css']
+
     _widget_type: ClassVar[Type[Model]] = HTML
 
     def _process_param_change(self, msg):
@@ -218,7 +219,9 @@ class Progress(ValueIndicator):
         bar will be indeterminate and animate depending on the active
         parameter.""")
 
-    _rename: ClassVar[Mapping[str, str | None]] = {'name': None}
+    width = param.Integer(default=300)
+
+    _stylesheets: ClassVar[List[str]] = [f'{CDN_DIST}css/progress.css']
 
     _widget_type: ClassVar[Type[Model]] = _BkProgress
 
@@ -227,6 +230,8 @@ class Progress(ValueIndicator):
         self.param.value.bounds = (-1, self.max)
 
     def __init__(self,**params):
+        if "sizing_mode" not in params:
+            params["sizing_mode"] = None
         super().__init__(**params)
         self._update_value_bounds()
 
@@ -262,7 +267,7 @@ class Number(ValueIndicator):
     title_size = param.String(default='18pt', doc="""
         The size of the title given by the name.""")
 
-    _rename: ClassVar[Mapping[str, str | None]] = {}
+    _rename: ClassVar[Mapping[str, str | None]] = {'name': 'name'}
 
     _source_transforms: ClassVar[Mapping[str, str | None]] = {
         'value': None, 'colors': None, 'default_color': None,
@@ -272,6 +277,11 @@ class Number(ValueIndicator):
 
     _widget_type: ClassVar[Type[Model]] = HTML
 
+    def __init__(self, **params):
+        if "sizing_mode" not in params:
+            params["sizing_mode"] = None
+        super().__init__(**params)
+
     def _process_param_change(self, msg):
         msg = super()._process_param_change(msg)
         font_size = msg.pop('font_size', self.font_size)
@@ -321,6 +331,11 @@ class String(ValueIndicator):
 
     _widget_type: ClassVar[Type[Model]] = HTML
 
+    def __init__(self, **params):
+        if "sizing_mode" not in params:
+            params["sizing_mode"] = None
+        super().__init__(**params)
+
     def _process_param_change(self, msg):
         msg = super()._process_param_change(msg)
         font_size = msg.pop('font_size', self.font_size)
@@ -496,6 +511,9 @@ class Dial(ValueIndicator):
 
     height = param.Integer(default=250, bounds=(1, None))
 
+    label_color = param.String(default='black', doc="""
+      Color for all extraneous labels.""")
+
     nan_format = param.String(default='-', doc="""
       How to format nan values.""")
 
@@ -530,12 +548,14 @@ class Dial(ValueIndicator):
         'annulus_width', 'format', 'background', 'needle_width',
         'tick_size', 'title_size', 'value_size', 'colors',
         'default_color', 'unfilled_color', 'height',
-        'width', 'nan_format', 'needle_color'
+        'width', 'nan_format', 'needle_color', 'label_color'
     ]
 
     _data_params: ClassVar[List[str]] = _manual_params
 
-    _rename: ClassVar[Mapping[str, str | None]] = {'background': 'background_fill_color'}
+    _rename: ClassVar[Mapping[str, str | None]] = {
+        'background': 'background_fill_color'
+    }
 
     def __init__(self, **params):
         super().__init__(**params)
@@ -545,7 +565,7 @@ class Dial(ValueIndicator):
     def _update_value_bounds(self):
         self.param.value.bounds = self.bounds
 
-    def _get_data(self):
+    def _get_data(self, properties):
         vmin, vmax = self.bounds
         value = self.value
         if value is None:
@@ -619,22 +639,22 @@ class Dial(ValueIndicator):
             'text': [self.name, value, min_value, max_value],
             'rot':  np.array([0, 0, tmin_angle, tmax_angle]),
             'size': [title_size, value_size, tick_size, tick_size],
-            'color': ['black', color, 'black', 'black']
+            'color': [self.label_color, color, self.label_color, self.label_color]
         }
         return annulus_data, needle_data, threshold_data, text_data
 
     def _get_model(self, doc, root=None, parent=None, comm=None):
-        params = self._process_param_change(self._init_params())
+        properties = self._get_properties(doc)
         model = figure(
             x_range=(-1,1), y_range=(-1,1), tools=[],
             outline_line_color=None, toolbar_location=None,
-            width=self.width, height=self.height, **params
+            width=self.width, height=self.height, **properties
         )
         model.xaxis.visible = False
         model.yaxis.visible = False
         model.grid.visible = False
 
-        annulus, needle, threshold, text = self._get_data()
+        annulus, needle, threshold, text = self._get_data(properties)
 
         # Draw annulus
         annulus_source = ColumnDataSource(data=annulus, name='annulus_source')
@@ -667,6 +687,9 @@ class Dial(ValueIndicator):
             angle='rot'
         )
 
+        # Apply theme
+        self._design.apply_bokeh_theme_to_model(model)
+
         if root is None:
             root = model
         self._models[root.ref['id']] = (model, parent)
@@ -685,7 +708,8 @@ class Dial(ValueIndicator):
                 needle_r.glyph.fill_color = event.new
         if not update_data:
             return
-        annulus, needle, threshold, labels = self._get_data()
+        properties = self._get_properties(doc)
+        annulus, needle, threshold, labels = self._get_data(properties)
         model.select(name='annulus_source').data.update(annulus)
         model.select(name='needle_source').data.update(needle)
         model.select(name='threshold_source').data.update(threshold)
@@ -763,7 +787,9 @@ class LinearGauge(ValueIndicator):
     _rerender_params = ['horizontal']
 
     _rename: ClassVar[Mapping[str, str | None]] = {
-        'background': 'background_fill_color', 'show_boundaries': None,
+        'background': 'background_fill_color',
+        'name': 'name',
+        'show_boundaries': None,
         'default_color': None
     }
 
@@ -807,7 +833,7 @@ class LinearGauge(ValueIndicator):
             intervals.append((1, self.unfilled_color))
         return intervals
 
-    def _get_data(self):
+    def _get_data(self, properties):
         vmin, vmax = self.bounds
         value = self.value
         interval = (vmax-vmin)
@@ -834,7 +860,7 @@ class LinearGauge(ValueIndicator):
         )
 
     def _get_model(self, doc, root=None, parent=None, comm=None):
-        params = self._process_param_change(self._init_params())
+        params = self._get_properties(doc)
         model = figure(
             outline_line_color=None, toolbar_location=None, tools=[],
             x_axis_location='above', y_axis_location='right', **params
@@ -851,8 +877,8 @@ class LinearGauge(ValueIndicator):
         self._update_axes(model)
         self._update_renderers(model)
         self._update_bounds(model)
-        if root is None:
-            root = model
+        self._design.apply_bokeh_theme_to_model(model)
+        root = root or model
         self._models[root.ref['id']] = (model, parent)
         return model
 
@@ -872,7 +898,8 @@ class LinearGauge(ValueIndicator):
 
     def _update_renderers(self, model):
         model.renderers = []
-        data, needle_data = self._get_data()
+        properties = self._get_properties(model.document)
+        data, needle_data = self._get_data(properties)
         bar_source = ColumnDataSource(data=data, name='bar_source')
         needle_source = ColumnDataSource(data=needle_data, name='needle_source')
         if self.horizontal:
@@ -932,7 +959,7 @@ class LinearGauge(ValueIndicator):
             model.yaxis.ticker = ticker
 
     def _update_figure(self, model):
-        params = self._process_param_change(self._init_params())
+        params = self._get_properties(model.document)
         if self.horizontal:
             params.update(width=self.height, height=self.width)
         else:
@@ -966,7 +993,8 @@ class LinearGauge(ValueIndicator):
                 self._update_title_size(model)
         if not update_data:
             return
-        data, needle_data = self._get_data()
+        properties = self._get_properties(model.document)
+        data, needle_data = self._get_data(properties)
         model.select(name='bar_source').data.update(data)
         model.select(name='needle_source').data.update(needle_data)
 
@@ -1012,6 +1040,10 @@ class Trend(SyncableData, Indicator):
     neg_color = param.String(RED, doc="""
       The color used to indicate a negative change.""")
 
+    sizing_mode = param.ObjectSelector(default=None, objects=[
+        'fixed', 'stretch_width', 'stretch_height', 'stretch_both',
+        'scale_width', 'scale_height', 'scale_both', None])
+
     title = param.String(doc="""The title or a short description of the card""")
 
     value = param.Parameter(default='auto', doc="""
@@ -1024,7 +1056,9 @@ class Trend(SyncableData, Indicator):
 
     _manual_params: ClassVar[List[str]] = ['data']
 
-    _rename: ClassVar[Mapping[str, str | None]] = {'data': None, 'selection': None}
+    _rename: ClassVar[Mapping[str, str | None]] = {
+        'data': None, 'name': 'name', 'selection': None
+    }
 
     _widget_type: ClassVar[Type[Model]] = _BkTrendIndicator
 
@@ -1096,10 +1130,10 @@ class ptqdm(_tqdm):
 
     def display(self, msg=None, pos=None, bar_style=None):
         super().display(msg, pos)
-        style = self._indicator.text_pane.style or {}
-        if "color" not in style:
+        styles = self._indicator.text_pane.styles or {}
+        if "color" not in styles:
             color = self.colour or 'black'
-            self._indicator.text_pane.style = dict(style, color=color)
+            self._indicator.text_pane.styles = dict(styles, color=color)
         if self.total is not None and self.n is not None:
             self._indicator.max = int(self.total) # Can be numpy.int64
             self._indicator.value = int(self.n)
@@ -1168,7 +1202,7 @@ class Tqdm(Indicator):
     _layouts: ClassVar[Dict[Type[Panel], str]] = {Row: 'row', Column: 'column'}
 
     _rename: ClassVar[Mapping[str, str | None]] = {
-        'value': None, 'min': None, 'max': None, 'text': None
+        'value': None, 'min': None, 'max': None, 'text': None, 'name': 'name'
     }
 
     def __init__(self, **params):
@@ -1218,8 +1252,7 @@ class Tqdm(Indicator):
         parent: Optional[Model] = None, comm: Optional[Comm] = None
     ) -> Model:
         model = self.layout._get_model(doc, root, parent, comm)
-        if root is None:
-            root = model
+        root = root or model
         self._models[root.ref['id']] = (model, parent)
         return model
 
