@@ -110,6 +110,9 @@ class FileDownload(Widget):
     label = param.String(default="Download file", doc="""
         The label of the download button""")
 
+    description = param.String(default=None, doc="""
+        An HTML string describing the function of this component.""")
+
     _clicks = param.Integer(default=0)
 
     _transfers = param.Integer(default=0)
@@ -136,7 +139,7 @@ class FileDownload(Widget):
     }
 
     _rename: ClassVar[Mapping[str, str | None]] = {
-        'callback': None, 'file': None, '_clicks': 'clicks', 'name': 'title'
+        'callback': None, 'file': None, '_clicks': 'clicks'
     }
 
     _widget_type: ClassVar[Type[Model]] = _BkFileDownload
@@ -258,7 +261,7 @@ class JSONEditor(Widget):
         of mode.""")
 
     mode = param.Selector(default='tree', objects=[
-        "tree", "view", "form", "code", "text", "preview"], doc="""
+        "tree", "view", "form", "text", "preview"], doc="""
         Sets the editor mode. In 'view' mode, the data and
         datastructure is read-only. In 'form' mode, only the value can
         be changed, the data structure is read-only. Mode 'code'
@@ -291,7 +294,9 @@ class JSONEditor(Widget):
     value = param.Parameter(default={}, doc="""
         JSON data to be edited.""")
 
-    _rename: ClassVar[Mapping[str, str | None]] = {'value': 'data'}
+    _rename: ClassVar[Mapping[str, str | None]] = {
+        'name': None, 'value': 'data'
+    }
 
     def _get_model(self, doc, root=None, parent=None, comm=None):
         if self._widget_type is None:
